@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.36 2004/01/01 18:53:28 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.37 2004/01/11 02:02:19 fuyu Exp $
  */
 
 #include "config.h"
@@ -1479,6 +1479,35 @@ bbs_thread_view_place_cursor_at_offset(BBSThreadView *view, int offset)
 }
 
 
+int
+bbs_thread_view_get_visible_offset(BBSThreadView *view)
+{
+  GdkRectangle visible_rect;
+  GtkTextIter iter;
+  gtk_text_view_get_visible_rect(GTK_TEXT_VIEW(view), &visible_rect);
+  gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(view), &iter,
+				     visible_rect.x, visible_rect.y);
+  return gtk_text_iter_get_offset(&iter);
+}
+
+
+void
+bbs_thread_view_make_offset_visible(BBSThreadView *view, int offset)
+{
+  GtkTextIter iter;
+  gboolean result;
+
+  g_return_if_fail(IS_BBS_THREAD_VIEW(view) && view->text_buffer != NULL)
+
+  CHECK_THREADS_ENTERED();
+
+  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &iter, offset);
+  gtk_text_buffer_place_cursor(view->text_buffer, &iter);
+  result = gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(view),
+					&iter, 0.0, TRUE, 0.0, 0.0);
+}
+
+
 /* Ϣ */
 static void
 bbs_thread_view_scroll_to_offsets(BBSThreadView *view,
