@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.h,v 1.19 2003/12/30 21:53:28 fuyu Exp $
+ * $Id: bbs_thread_view.h,v 1.20 2004/01/11 02:02:19 fuyu Exp $
  */
 
 #ifndef _BBS_THREAD_VIEW_H_
@@ -189,6 +189,8 @@ GtkTextMark *bbs_thread_view_create_mark_at_offset(BBSThreadView *view,
 						   int offset);
 void bbs_thread_view_place_cursor_at_offset(BBSThreadView *view, int offset);
 
+int bbs_thread_view_get_visible_offset(BBSThreadView *view);
+void bbs_thread_view_make_offset_visible(BBSThreadView *view, int offset);
 
 gboolean bbs_thread_view_find(BBSThreadView *view, const gchar *key,
 			      BBSThreadViewSearchDirection direction,
