@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: board_properties.c,v 1.9 2003/12/30 21:53:28 fuyu Exp $
+ * $Id: board_properties.c,v 1.11 2004/01/11 03:03:58 fuyu Exp $
  */
 
 #include "config.h"
@@ -452,7 +452,9 @@ build_board_info_frame(OchushaApplication *application,
   GtkWidget *tmp_widget;
   GtkWidget *board_frame = gtk_frame_new(NULL);
 
+#if 0	/* themeˤޤ٤ */
   gtk_frame_set_shadow_type(GTK_FRAME(board_frame), GTK_SHADOW_ETCHED_OUT);
+#endif
   gtk_widget_show(board_frame);
 
   table = gtk_table_new(2, 5, FALSE);
@@ -523,12 +525,12 @@ build_thread_view_setting_frame(OchushaApplication *application,
 
   gtk_widget_show(hbox);
 
-  tmp_widget = gtk_check_button_new_with_label(_("Use board specific setting for a bone."));
+  tmp_widget = gtk_check_button_new_with_label(_("Use Board Specific Setting for A Bone."));
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
   states->board_local_a_bone_button = GTK_TOGGLE_BUTTON(tmp_widget);
 
-  tmp_widget = gtk_check_button_new_with_label(_("A bone by name field."));
+  tmp_widget = gtk_check_button_new_with_label(_("A Bone by Name Field."));
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
   states->a_bone_by_name_button = GTK_TOGGLE_BUTTON(tmp_widget);
@@ -538,7 +540,7 @@ build_thread_view_setting_frame(OchushaApplication *application,
   gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, TRUE, TRUE, 0);
   states->a_bone_by_name_pattern = REGEX_EDITOR(tmp_widget);
 
-  tmp_widget = gtk_check_button_new_with_label(_("A bone by ID."));
+  tmp_widget = gtk_check_button_new_with_label(_("A Bone by ID."));
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
   states->a_bone_by_id_button = GTK_TOGGLE_BUTTON(tmp_widget);
@@ -548,7 +550,7 @@ build_thread_view_setting_frame(OchushaApplication *application,
   gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, TRUE, TRUE, 0);
   states->a_bone_by_id_pattern = REGEX_EDITOR(tmp_widget);
 
-  tmp_widget = gtk_check_button_new_with_label(_("A bone by content."));
+  tmp_widget = gtk_check_button_new_with_label(_("A Bone by Content."));
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
   states->a_bone_by_content_button = GTK_TOGGLE_BUTTON(tmp_widget);
@@ -571,13 +573,15 @@ build_filter_setting_frame(OchushaApplication *application,
   GtkWidget *vbox;
   GtkWidget *tmp_widget;
 
+#if 0	/* themeǤˤޤ٤ */
   gtk_frame_set_shadow_type(GTK_FRAME(filter_frame), GTK_SHADOW_ETCHED_OUT);
+#endif
   gtk_widget_show(filter_frame);
   vbox = gtk_vbox_new(FALSE, 0);
   gtk_widget_show(vbox);
   gtk_container_add(GTK_CONTAINER(filter_frame), vbox);
 
-  tmp_widget = gtk_check_button_new_with_label(_("Use this setting as the default setting."));
+  tmp_widget = gtk_check_button_new_with_label(_("Use This Setting as the Default Setting."));
   states->use_as_default_button = GTK_TOGGLE_BUTTON(tmp_widget);
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
