@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.53 2004/01/01 23:05:58 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.68 2004/01/11 03:03:58 fuyu Exp $
  */
 
 #include "config.h"
@@ -43,6 +43,7 @@
 #include "response_editor.h"
 #include "text_search_window.h"
 #include "ugly_gtk2utils.h"
+#include "virtual_board.h"
 
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -83,28 +84,37 @@ static void toggle_hide_board_cb(BoardlistView *view,
 static void kill_category_cb(BoardlistView *view,
 			     OchushaBoardCategory *category,
 			     OchushaApplication *application);
-static void kill_board_cb(BoardlistView *view, OchushaBulletinBoard *board,
+static void kill_board_cb(BoardlistView *view, OchushaBoardCategory *category,
+			  OchushaBulletinBoard *board,
 			  OchushaApplication *application);
 static void copy_board_url_cb(BoardlistView *view, OchushaBulletinBoard *board,
 			      OchushaApplication *application);
+static void uri_dropped_cb(BoardlistView *view, OchushaBoardCategory *category,
+			   OchushaBulletinBoard *board, const char *uri,
+			   GtkTreeViewDropPosition pos,
+			   OchushaApplication *application);
 
+static OchushaBulletinBoard *board_new_cb(OchushaBBSTable *table,
+					  int bbs_type, const gchar *name,
+					  const char *base_url,
+					  OchushaApplication *application);
 static void boardlist_read_category_element_cb(OchushaBBSTable *table,
 					       OchushaBoardCategory *category,
 					       GHashTable *cat_attributes,
-					       OchushaApplication *appl);
+					       OchushaApplication *app);
 static void boardlist_read_board_element_cb(OchushaBBSTable *table,
 					    OchushaBulletinBoard *board,
 					    GHashTable *cat_attributes,
-					    OchushaApplication *appl);
+					    OchushaApplication *app);
 
 static void boardlist_write_category_element_cb(OchushaBBSTable *table,
 						OchushaBoardCategory *category,
 						gzFile *boardlist_xml,
-						OchushaApplication *appl);
+						OchushaApplication *app);
 static void boardlist_write_board_element_cb(OchushaBBSTable *table,
 					     OchushaBulletinBoard *board,
 					     gzFile *boardlist_xml,
-					     OchushaApplication *appl);
+					     OchushaApplication *app);
 
 static void write_threadlist(gpointer data, gpointer user_data);
 
@@ -163,12 +173,6 @@ static pthread_mutex_t board_list_lock;
     }
 
 
-typedef struct _BoardCategoryGUIInfo
-{
-  gboolean expanded;
-} BoardCategoryGUIInfo;
-
-
 static GQuark category_gui_info_id;
 static GQuark application_id;
 
@@ -217,6 +221,42 @@ open_threadlist_view_cb(GtkWidget *selector, OchushaBulletinBoard *board,
 static guint new_thread_netstat_id;
 
 
+static void
+ensure_my_folder(OchushaApplication *application)
+{
+  BoardCategoryGUIInfo *info;
+  if (application->my_folder == NULL)
+    {
+      application->my_folder = ochusha_board_category_new(_("My Folder"));
+      info = ensure_board_category_info(application->my_folder);
+      info->category_type = CATEGORY_TYPE_OCHUSHA_MY_FOLDER;
+      ochusha_bbs_table_prepend_category(application->table,
+					 application->my_folder);
+    }
+
+  if (application->favorites == NULL)
+    {
+      /* äURLǤܤ */
+      application->favorites = virtual_board_new(_("Favorites"),
+						 "http:///favorites",
+						 application);
+      ochusha_board_category_add_board(application->my_folder,
+				       application->favorites);
+    }
+
+#if 0
+  if (application->all_threads == NULL)
+    {
+      application->all_threads = virtual_board_new(_("All Threads Read"),
+						   "http:///all-threads",
+						   application);
+      ochusha_board_category_add_board(application->my_folder,
+				       application->all_threads);
+    }
+#endif
+}
+
+
 void
 initialize_boardlist(OchushaApplication *application)
 {
@@ -248,6 +288,10 @@ initialize_boardlist(OchushaApplication *application)
 		   G_CALLBACK(kill_board_cb), application);
   g_signal_connect(G_OBJECT(application->boardlist_view), "copy_board_url",
 		   G_CALLBACK(copy_board_url_cb), application);
+
+  g_signal_connect(G_OBJECT(application->boardlist_view), "uri_dropped",
+		   G_CALLBACK(uri_dropped_cb), application);
+
 #if TRACE_MEMORY_USAGE
   g_signal_connect(G_OBJECT(application->boardlist_view), "destroy",
 		   G_CALLBACK(gtk_widget_destroyed),
@@ -261,8 +305,10 @@ initialize_boardlist(OchushaApplication *application)
   scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
 				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+#if 0	/* themeǤˤޤ٤ */
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window),
 				      GTK_SHADOW_ETCHED_IN);
+#endif
   gtk_container_add(GTK_CONTAINER(scrolled_window),
 		    application->boardlist_view);
 
@@ -288,7 +334,7 @@ initialize_boardlist(OchushaApplication *application)
 
   gtk_toolbar_prepend_space(toolbar);
   gtk_toolbar_insert_stock(toolbar, GTK_STOCK_NEW,
-			   _("Make a new thread at the current board"),
+			   _("Make a New Thread at the Current Board"),
 			   "make_new_thread_at_current_board",
 			   GTK_SIGNAL_FUNC(make_new_thread_button_cb),
 			   application,
@@ -296,7 +342,7 @@ initialize_boardlist(OchushaApplication *application)
 
   gtk_toolbar_prepend_space(toolbar);
   gtk_toolbar_insert_stock(toolbar, GTK_STOCK_PROPERTIES,
-			   _("Open the current board's properties"),
+			   _("Open the Current Board's Properties"),
 			   "open_the_current_board_properties",
 			   GTK_SIGNAL_FUNC(open_board_properties_button_cb),
 			   application,
@@ -306,7 +352,7 @@ initialize_boardlist(OchushaApplication *application)
   gtk_widget_show(filter_button);
   application->filter_button = GTK_TOGGLE_BUTTON(filter_button);
   gtk_toolbar_prepend_widget(toolbar, filter_button,
-			     _("Apply filter to the current threadlist"),
+			     _("Apply Filter to the Current Threadlist"),
 			     "apply_filter_to_the_current_threadlist");
   g_signal_connect(G_OBJECT(filter_button), "toggled",
 		   G_CALLBACK(filter_toggled_cb), application);
@@ -317,17 +363,18 @@ initialize_boardlist(OchushaApplication *application)
 #endif
 
   gtk_toolbar_prepend_space(toolbar);
-  gtk_toolbar_insert_stock(toolbar, GTK_STOCK_REFRESH,
-			   _("Refresh current threadlist"),
-			   "refresh_current_threadlist",
-			   GTK_SIGNAL_FUNC(refresh_board_button_cb),
+  gtk_toolbar_insert_stock(toolbar, GTK_STOCK_FIND,
+			   _("Start Text Search on the Current Threadlist"),
+			   "start_text_search_on_threadlist",
+			   GTK_SIGNAL_FUNC(start_threadlist_search_button_cb),
 			   application,
 			   0);
 
-  gtk_toolbar_insert_stock(toolbar, GTK_STOCK_FIND,
-			   _("Start text search on current threadlist"),
-			   "start_text_search_on_threadlist",
-			   GTK_SIGNAL_FUNC(start_threadlist_search_button_cb),
+  gtk_toolbar_prepend_space(toolbar);
+  gtk_toolbar_insert_stock(toolbar, GTK_STOCK_REFRESH,
+			   _("Refresh the Current Threadlist"),
+			   "refresh_current_threadlist",
+			   GTK_SIGNAL_FUNC(refresh_board_button_cb),
 			   application,
 			   0);
 
@@ -364,7 +411,9 @@ initialize_boardlist(OchushaApplication *application)
 
   application->table = ochusha_bbs_table_new();
 
-  /* γĥǥ¸ */
+  g_signal_connect(G_OBJECT(application->table),
+		   "board_new",
+		   G_CALLBACK(board_new_cb), application);
   g_signal_connect(G_OBJECT(application->table),
 		   "boardlist_read_category_element",
 		   G_CALLBACK(boardlist_read_category_element_cb),
@@ -406,7 +455,7 @@ trace_free(gpointer pointer)
 #endif
 
 
-static BoardCategoryGUIInfo *
+BoardCategoryGUIInfo *
 ensure_board_category_info(OchushaBoardCategory *category)
 {
   BoardCategoryGUIInfo *info = g_object_get_qdata(G_OBJECT(category),
@@ -435,6 +484,18 @@ refresh_boardlist(OchushaApplication *application)
 }
 
 
+static void
+update_board_list(gpointer list_data, gpointer user_data)
+{
+  if (g_slist_find(board_list, list_data) == NULL)
+    {
+      OchushaBulletinBoard *board = OCHUSHA_BULLETIN_BOARD(list_data);
+      if (board->thread_list != NULL)
+	board_list = g_slist_append(board_list, board);
+    }
+}
+
+
 void
 finalize_boardlist(OchushaApplication *application)
 {
@@ -468,6 +529,13 @@ finalize_boardlist(OchushaApplication *application)
 	fprintf(stderr, "Couldn't write boardlist.xml.\n");
     }
 
+  if (IS_VIRTUAL_BOARD(application->favorites))
+    {
+      VirtualBoard *favorites = VIRTUAL_BOARD(application->favorites);
+      g_slist_foreach(favorites->board_list, update_board_list, NULL);
+      write_threadlist(favorites, application);
+    }
+
   g_slist_foreach(board_list, write_threadlist, application);
 }
 
@@ -489,7 +557,7 @@ make_new_thread(OchushaApplication *application)
   contents_window = PANED_NOTEBOOK(application->contents_window);
 
   board = (OchushaBulletinBoard *)paned_notebook_get_current_item(contents_window);
-  if (board == NULL)
+  if (board == NULL || IS_VIRTUAL_BOARD(board))
     return;
 
   if (!ochusha_bulletin_board_is_new_thread_supported(board))
@@ -692,6 +760,11 @@ text_search_window_query_changed_cb(TextSearchWindow *window, const gchar *key,
 
   MAIN_THREAD_EVENT_CALLBACK();
 
+  application->threadlist_search_direction = direction;
+  application->threadlist_search_enable_wrap = enable_wrap;
+  application->threadlist_search_match_case = match_case;
+  application->threadlist_search_use_regexp = use_regexp;
+
   if (search_query_data.key != NULL)
     G_FREE(search_query_data.key);
 
@@ -768,6 +841,7 @@ text_search_window_find_next_cb(TextSearchWindow *window, const gchar *key,
 static void
 setup_threadlist_search_window(OchushaApplication *application)
 {
+  TextSearchWindow *window;
   if (search_query_data.key != NULL)
     {
       G_FREE(search_query_data.key);
@@ -775,9 +849,18 @@ setup_threadlist_search_window(OchushaApplication *application)
     }
 
   application->threadlist_search_window = text_search_window_new();
-  text_search_window_set_enable_incremental_search(
-		TEXT_SEARCH_WINDOW(application->threadlist_search_window),
-		TRUE);
+
+  window = TEXT_SEARCH_WINDOW(application->threadlist_search_window);
+  text_search_window_set_enable_incremental_search(window, TRUE);
+  text_search_window_set_direction(window,
+				   application->threadlist_search_direction);
+  text_search_window_set_enable_wrap(window,
+				     application->threadlist_search_enable_wrap);
+  text_search_window_set_match_case(window,
+				    application->threadlist_search_match_case);
+  text_search_window_set_use_regexp(window,
+				    application->threadlist_search_use_regexp);
+
   g_signal_connect(application->threadlist_search_window, "response",
 		   G_CALLBACK(text_search_window_response_cb),
 		   application);
@@ -793,7 +876,7 @@ setup_threadlist_search_window(OchushaApplication *application)
 		   &application->threadlist_search_window);
 #endif
   gtk_window_set_title(GTK_WINDOW(application->threadlist_search_window),
-		       _("Find in the current threadlist"));
+		       _("Find in the Current Threadlist"));
   gtk_window_set_transient_for(
 			GTK_WINDOW(application->threadlist_search_window),
 			application->top_level);
@@ -862,10 +945,9 @@ start_current_threadlist_search(OchushaApplication *application)
   if (application->threadlist_search_window == NULL)
     setup_threadlist_search_window(application);
 
+  gtk_widget_show(application->threadlist_search_window);
   text_search_window_set_key_selected(
 		TEXT_SEARCH_WINDOW(application->threadlist_search_window));
-  gtk_widget_show(application->threadlist_search_window);
-  gtk_widget_grab_focus(application->threadlist_search_window);
 }
 
 
@@ -957,12 +1039,15 @@ item_view_required_cb(PanedNotebook *paned_notebook, gpointer item,
       return;
     }
 
-  BOARD_LIST_LOCK
-  {
-    if (g_slist_find(board_list, board) == NULL)
-      board_list = g_slist_append(board_list, board);
-  }
-  BOARD_LIST_UNLOCK;
+  if (!IS_VIRTUAL_BOARD(board))
+    {
+      BOARD_LIST_LOCK
+      {
+	if (g_slist_find(board_list, board) == NULL)
+	  board_list = g_slist_append(board_list, board);
+      }
+      BOARD_LIST_UNLOCK;
+    }
 
   gtk_widget_show(*item_view);
   *title = board->name;
@@ -1006,7 +1091,7 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
       OCHU_THREADS_LEAVE();
       ochusha_async_buffer_suspend(buffer);
       ochusha_async_buffer_terminate(buffer);
-      ochusha_async_buffer_resume(buffer);
+      /* ochusha_async_buffer_resume(buffer); */
       OCHU_THREADS_ENTER();
     }
 
@@ -1107,10 +1192,32 @@ page_switched_cb(PanedNotebook *paned_notebook,
       tmp_widget = gtk_bin_get_child(GTK_BIN(tmp_widget));
       threadlist_view_set_default_open_in_tab(THREADLIST_VIEW(tmp_widget),
 					application->default_open_in_tab);
+      threadlist_view_set_remove_hidden_thread(THREADLIST_VIEW(tmp_widget),
+					       application->hide_hidden_threads);
     }
 }
 
 
+static OchushaBulletinBoard *
+board_new_cb(OchushaBBSTable *table, int bbs_type, const gchar *name,
+	     const char *base_url, OchushaApplication *application)
+{
+  if (bbs_type == OCHUSHA_BBS_TYPE_VIRTUAL_BOARD)
+    {
+      OchushaBulletinBoard *board
+	= virtual_board_new(name, base_url, application);
+      if (strcmp(base_url, "http:///favorites") == 0)
+	application->favorites = board;
+#if 0
+      else if (strcmp(base_url, "http:///all-threads") == 0)
+	application->all_threads = board;
+#endif
+      return board;
+    }
+  return NULL;	/* ¾ΤϤǤ */
+}
+
+
 static void
 boardlist_read_category_element_cb(OchushaBBSTable *table,
 				   OchushaBoardCategory *category,
@@ -1121,6 +1228,14 @@ boardlist_read_category_element_cb(OchushaBBSTable *table,
 
   info->expanded = ochusha_utils_get_attribute_boolean(category_attributes,
 						       "expanded");
+  info->category_type = ochusha_utils_get_attribute_int(category_attributes,
+							"category_type");
+  if (info->category_type == CATEGORY_TYPE_OCHUSHA_MY_FOLDER)
+    {
+      application->my_folder = category;
+      ochusha_board_category_set_name(category, _("My Folder"));
+    }
+
   return;
 }
 
@@ -1184,10 +1299,22 @@ boardlist_read_board_element_cb(OchushaBBSTable *table,
       if ((category)->attribute)					\
 	{								\
 	  gzprintf(gzfile,						\
-		  "      <attribute name=\"" #attribute	"\">\n"		\
-		  "        <boolean val=\"%s\"/>\n"			\
-		  "      </attribute>\n",				\
-		  (category)->attribute ? "true" : "false");		\
+		   "      <attribute name=\"" #attribute	"\">\n"	\
+		   "        <boolean val=\"%s\"/>\n"			\
+		   "      </attribute>\n",				\
+		   (category)->attribute ? "true" : "false");		\
+	}								\
+    } while (0)
+
+#define OUTPUT_CATEGORY_ATTRIBUTE_INT(gzfile, category, attribute)	\
+  do									\
+    {									\
+      if ((category)->attribute)					\
+	{								\
+	  gzprintf(gzfile,						\
+		   "      <attribute name=\"" #attribute	"\">\n"	\
+		   "        <int val=\"%d\"/>\n"			\
+		   "      </attribute>\n", (category)->attribute);	\
 	}								\
     } while (0)
 
@@ -1201,6 +1328,7 @@ boardlist_write_category_element_cb(OchushaBBSTable *table,
   BoardCategoryGUIInfo *info = ensure_board_category_info(category);
 
   OUTPUT_CATEGORY_ATTRIBUTE_BOOLEAN(boardlist_xml, info, expanded);
+  OUTPUT_CATEGORY_ATTRIBUTE_INT(boardlist_xml, info, category_type);
 }
 
 
@@ -1295,13 +1423,13 @@ write_threadlist(gpointer data, gpointer user_data)
   /* XXX: 顼 */
   if (!ochusha_bulletin_board_write_threadlist_xml(board, &application->config,
 						application->session_subdir))
-    fprintf(stderr, "Couldn't write threadlist2.xml.\n");
+    fprintf(stderr, "Couldn't write threadlist.xml.\n");
 
   if (application->session_subdir != NULL
       && !ochusha_bulletin_board_write_threadlist_xml(board,
 						      &application->config,
 						      NULL))
-    fprintf(stderr, "Couldn't write threadlist2.xml.\n");
+    fprintf(stderr, "Couldn't write threadlist.xml.\n");
 }
 
 
@@ -1359,10 +1487,11 @@ kill_category_cb(BoardlistView *view, OchushaBoardCategory *category,
 
 
 static void
-kill_board_cb(BoardlistView *view, OchushaBulletinBoard *board,
-	      OchushaApplication *application)
+kill_board_cb(BoardlistView *view, OchushaBoardCategory *category,
+	      OchushaBulletinBoard *board, OchushaApplication *application)
 {
-  board->killed = TRUE;
+  if (category != NULL)
+    ochusha_board_category_remove_board(category, board);
 }
 
 
@@ -1375,6 +1504,72 @@ copy_board_url_cb(BoardlistView *view, OchushaBulletinBoard *board,
 }
 
 
+static void
+uri_dropped_cb(BoardlistView *view, OchushaBoardCategory *category,
+	       OchushaBulletinBoard *board, const char *uri,
+	       GtkTreeViewDropPosition pos,
+	       OchushaApplication *application)
+{
+  OchushaBulletinBoard *dropped_board = NULL;
+  char *dropped_thread_id = NULL;
+  gboolean is_kako_html = FALSE;
+  BoardCategoryGUIInfo *category_info;
+#if 0
+  fprintf(stderr, "uri_dropped_cb: category(%s)/board(%s) uri=\"%s\"\n",
+	  ochusha_board_category_get_name(category),
+	  ochusha_bulletin_board_get_name(board), uri);
+#endif
+  if (!OCHUSHA_IS_BOARD_CATEGORY(category))
+    return;
+
+  category_info = ensure_board_category_info(category);
+  if (category_info->category_type == CATEGORY_TYPE_BBS_MENU_CATEGORY)
+    return;
+
+  if (ochusha_bbs_table_check_url(application->table, uri,
+				  &dropped_board, NULL, NULL,
+				  &dropped_thread_id, NULL, NULL,
+				  &is_kako_html))
+    {
+      if (dropped_board == NULL)
+	return;
+
+      if (dropped_thread_id == NULL)
+	{
+	  /* Ĥɥåפ줿 */
+	  /* 褺categoryΥĤͤù */
+	  OchushaBulletinBoard *tmp_board
+	    = ochusha_board_category_lookup_board_by_url(category,
+							 dropped_board->base_url);
+	  if (tmp_board != NULL)
+	    {
+	      g_object_ref(G_OBJECT(tmp_board));
+	      ochusha_board_category_remove_board(category, tmp_board);
+	      boardlist_view_remove_board(view, category, tmp_board);
+	    }
+	  
+	  ochusha_board_category_add_board(category, dropped_board);
+
+	  if (tmp_board != NULL)
+	    g_object_unref(G_OBJECT(tmp_board));
+	  
+	  boardlist_view_add_board(view, dropped_board,
+				   category, board);
+	}
+      else
+	{
+	  /* 줬ɥåפ줿 */
+	  OchushaBBSThread *dropped_thread
+	    = ochusha_bulletin_board_lookup_bbs_thread_by_id(dropped_board,
+							     dropped_thread_id);
+	  if (dropped_thread != NULL)
+	    bookmark_thread(board, dropped_thread, application);
+	  G_FREE(dropped_thread_id);
+	}
+    }
+}
+
+
 static gboolean
 prepare_category(OchushaBoardCategory *category, gpointer user_data)
 {
@@ -1506,6 +1701,7 @@ restore_expanded_state(OchushaBoardCategory *category, BoardlistView *view)
 static void
 setup_boardlist(OchushaApplication *application)
 {
+  ensure_my_folder(application);
   boardlist_view_open(BOARDLIST_VIEW(application->boardlist_view),
 		      application->table->category_list,
 		      application->hide_hidden_boardlist_items);
@@ -1516,6 +1712,17 @@ setup_boardlist(OchushaApplication *application)
 
   gtk_widget_show_all(application->boardlist_view);
   gtk_widget_show_all(application->contents_window);
+
+#if 0
+  if (IS_VIRTUAL_BOARD(application->favorites)
+      && application->favorites->thread_list == NULL)
+    {
+      ochusha_bulletin_board_read_threadlist_xml(application->favorites,
+						 &application->config,
+						 application->session_subdir,
+						 FALSE);
+    }
+#endif
 }
 
 
@@ -1576,6 +1783,15 @@ new_thread_dialog_response_cb(GtkWidget *dialog, int response_id,
 
   MAIN_THREAD_EVENT_CALLBACK();
 
+  if (response_id == GTK_RESPONSE_REJECT)
+    {
+      ResponseEditor *editor = g_object_get_data(G_OBJECT(dialog), "editor");
+      response_editor_clear_response(editor);
+
+      ENTER_MAIN_EVENT_LOOP();
+      return;
+    }
+
   if (message_id != 0)
     {
       gtk_statusbar_remove(application->statusbar,
@@ -1715,6 +1931,7 @@ create_new_thread_dialog(OchushaApplication *application,
 					 application->top_level,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					 GTK_STOCK_CLEAR, GTK_RESPONSE_REJECT,
 					 GTK_STOCK_OK, GTK_RESPONSE_OK,
 					 NULL);
   gtk_window_set_position(GTK_WINDOW(new_thread_dialog), GTK_WIN_POS_CENTER);
