@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.c,v 1.22 2004/01/04 06:45:09 fuyu Exp $
+ * $Id: boardlist_view.c,v 1.27 2004/01/10 23:37:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -133,6 +133,8 @@ enum {
   DO_ON_SELECTION_SIGNAL,
   DO_ON_ITEM_AT_CURSOR_SIGNAL,
 
+  URI_DROPPED_SIGNAL,
+
   LAST_SIGNAL
 };
 
@@ -157,7 +159,7 @@ enum {
 
 static GtkTreeViewClass *parent_class = NULL;
 static int boardlist_view_signals[LAST_SIGNAL] =
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static GtkItemFactory *popup_menu_item_factory = NULL;
 static GtkItemFactory *popup_menu_item_factory2 = NULL;
 static GQuark node_info_id;
@@ -442,8 +444,9 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(BoardlistViewClass, kill_board),
 		 NULL, NULL,
-		 ochusha_marshal_VOID__OBJECT,
-		 G_TYPE_NONE, 1,
+		 ochusha_marshal_VOID__OBJECT_OBJECT,
+		 G_TYPE_NONE, 2,
+		 OCHUSHA_TYPE_BOARD_CATEGORY,
 		 OCHUSHA_TYPE_BULLETIN_BOARD);
   boardlist_view_signals[COPY_BOARD_URL_SIGNAL] =
     g_signal_new("copy_board_url",
@@ -493,6 +496,19 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 		 G_TYPE_NONE, 1,
 		 G_TYPE_INT);
 
+  boardlist_view_signals[URI_DROPPED_SIGNAL] =
+    g_signal_new("uri_dropped",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		 G_STRUCT_OFFSET(BoardlistViewClass, uri_dropped),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__OBJECT_OBJECT_STRING_ENUM,
+		 G_TYPE_NONE, 4,
+		 OCHUSHA_TYPE_BOARD_CATEGORY,
+		 OCHUSHA_TYPE_BULLETIN_BOARD,
+		 G_TYPE_STRING,
+		 GTK_TYPE_TREE_VIEW_DROP_POSITION);
+
 
   /* Key bindings */
   gtk_binding_entry_add_signal(binding_set, GDK_o, 0, "do_on_selection", 1,
@@ -584,7 +600,6 @@ enum
 
 enum
 {
-  TARGET_BOARDLIST_INTERNAL,
   TARGET_URI_LIST,
   TARGET_UTF8_STRING,
   TARGET_STRING,
@@ -593,31 +608,117 @@ enum
 };
 
 
-#define BOARDLIST_DND_INTERNAL_DATA	"Ochusha::Boardlist::DnD"
-
 static void
 drag_data_received_cb(GtkWidget *widget, GdkDragContext *context,
 		      int x, int y, GtkSelectionData *selection_data,
 		      guint info, guint time, BoardlistView *view)
 {
+  int row_type = 0;
+  gpointer data = NULL;
+  GtkTreePath *path;
+  GtkTreeViewDropPosition pos;
+  OchushaBoardCategory *category = NULL;
+  OchushaBulletinBoard *board = NULL;
+  GtkTreeIter iter;
+  const char *tmp_pos;
+  char uri_buffer[PATH_MAX];
+
 #if 0
   fprintf(stderr, "drag_data_received_cb: is_source=%d actions=0x%x suggested_action=0x%x action=0x%x time=%d\n",
 	  context->is_source, context->actions, context->suggested_action,
 	  context->action, time);
-  fprintf(stderr, "view->last_event_time=%d\n", view->last_event_time);
 #endif
+  if (!gtk_tree_view_get_dest_row_at_pos(&view->view, x, y, &path, &pos))
+    {
+      fprintf(stderr, "Couldn't know drop position.\n");
+      return;
+    }
+
+  if (!gtk_tree_model_get_iter(view->model, &iter, path))
+    {
+      fprintf(stderr, "Couldn't get GtkTreeIter for path.\n");
+      goto done;
+    }
+
+  gtk_tree_model_get(view->model, &iter,
+		     BOARDLIST_ROW_TYPE, &row_type,
+		     BOARDLIST_DATA, &data,
+		     -1);
+
+  switch (row_type)
+    {
+    case BOARDLIST_CATEGORY_ROW:
+      category = OCHUSHA_BOARD_CATEGORY(data);
+      break;
+
+    case BOARDLIST_BOARD_ROW:
+      board = OCHUSHA_BULLETIN_BOARD(data);
+      if (gtk_tree_path_up(path)
+	  && gtk_tree_model_get_iter(view->model, &iter, path))
+	{
+	  gtk_tree_model_get(view->model, &iter,
+			     BOARDLIST_DATA, &category, -1);
+	}
+      break;
+    }
+
+  tmp_pos = (const char *)selection_data->data;
+  while (tmp_pos != NULL)
+    {
+      size_t len;
+      const char *tail_pos = strstr(tmp_pos, "\r\n");
+      if (tail_pos == NULL)
+	break;
+
+      len = tail_pos - tmp_pos;
+      if (len > 0 && len < PATH_MAX)
+	{
+	  memcpy(uri_buffer, tmp_pos, len);
+	  uri_buffer[len] = '\0';
+	  g_signal_emit(G_OBJECT(view),
+			boardlist_view_signals[URI_DROPPED_SIGNAL],
+			0,
+			category, board, uri_buffer, pos);
+	}
+      tmp_pos = tail_pos + 2;
+    }
+
+  gtk_drag_finish(context, TRUE, FALSE, time);
+
+ done:
+  if (path != NULL)
+    gtk_tree_path_free(path);
 }
 
 
-static void
-collect_drag_sources(GtkTreeModel *model, GtkTreePath *path,
-		     GtkTreeIter *iter, GSList **list_p)
+typedef struct _CollectArgs
 {
+  GtkTreeModel *model;
+  GSList **list_p;
+} CollectArgs;
+
+
+static void
+collect_drag_sources(gpointer list_data, gpointer user_data)
+{
+  GtkTreePath *path = (GtkTreePath *)list_data;
+  CollectArgs *args = (CollectArgs *)user_data;
+  GtkTreeModel *model = args->model;
+  GSList **list_p = args->list_p;
+  GtkTreeIter iter;
   gpointer data = NULL;
-  gtk_tree_model_get(model, iter, BOARDLIST_DATA, &data, -1);
-  g_return_if_fail(data != NULL);
+
+  if (!gtk_tree_model_get_iter(model, &iter, path))
+    goto done;
+
+  gtk_tree_model_get(model, &iter, BOARDLIST_DATA, &data, -1);
+  if (data == NULL)
+    goto done;
 
   *list_p = g_slist_append(*list_p, data);
+
+ done:
+  gtk_tree_path_free(path);
 }
 
 
@@ -686,28 +787,16 @@ drag_data_get_cb(GtkWidget *widget, GdkDragContext *context,
 		 BoardlistView *view)
 {
   GSList *data_list = NULL;
-  gtk_tree_selection_selected_foreach(view->selection,
-			(GtkTreeSelectionForeachFunc)collect_drag_sources,
-			&data_list);
-  if (view->last_drag_event_data != NULL)
-    {
-      g_slist_free(view->last_drag_event_data);
-      view->last_drag_event_data = NULL;
-    }
+  GList *rows = gtk_tree_selection_get_selected_rows(view->selection, NULL);
+  CollectArgs args = { view->model, &data_list };
+
+  g_list_foreach(rows, collect_drag_sources, &args);
+  g_list_free(rows);
 
   if (data_list != NULL)
     {
       switch (info)
 	{
-	case TARGET_BOARDLIST_INTERNAL:
-	  view->last_drag_event_data = data_list;
-	  view->last_event_time = time;
-	  gtk_selection_data_set(selection_data, selection_data->target,
-				 8, BOARDLIST_DND_INTERNAL_DATA,
-				 strlen(BOARDLIST_DND_INTERNAL_DATA));
-	  data_list = NULL;
-	  break;
-
 	case TARGET_URI_LIST:
 	  {
 	    char *uri_list = NULL;
@@ -756,13 +845,12 @@ boardlist_view_init(BoardlistView *view)
   GtkTreeViewColumn *column;
   static const GtkTargetEntry drop_types[] =
     {
-      { "ochusha/boardlist-internal", 0, TARGET_BOARDLIST_INTERNAL }
+      { "text/uri-list", 0, TARGET_URI_LIST },
     };
   static int n_drop_types = sizeof(drop_types)/sizeof(drop_types[0]);
 
   static const GtkTargetEntry drag_types[] =
     {
-      { "ochusha/boardlist-internal", 0, TARGET_BOARDLIST_INTERNAL },
       { "text/uri-list", 0, TARGET_URI_LIST },
       { "UTF8_STRING", 0, TARGET_UTF8_STRING },
       { "STRING", 0, 0 },
@@ -794,14 +882,14 @@ boardlist_view_init(BoardlistView *view)
   /* D&Dμ¸ */
   gtk_tree_view_enable_model_drag_dest(&view->view,
 				       drop_types, n_drop_types,
-				       GDK_ACTION_MOVE);
+				       GDK_ACTION_COPY);
   g_signal_connect(view, "drag_data_received",
 		   G_CALLBACK(drag_data_received_cb), view);
 
   gtk_tree_view_enable_model_drag_source(&view->view,
 					 GDK_BUTTON1_MASK,
 					 drag_types, n_drag_types,
-					 GDK_ACTION_COPY | GDK_ACTION_MOVE);
+					 GDK_ACTION_COPY);
   g_signal_connect(view, "drag_data_get",
 		   G_CALLBACK(drag_data_get_cb), view);
 }
@@ -836,12 +924,6 @@ boardlist_view_destroy(GtkObject *object)
       view->last_event_path = NULL;
     }
 
-  if (view->last_drag_event_data != NULL)
-    {
-      g_slist_free(view->last_drag_event_data);
-      view->last_drag_event_data = NULL;
-    }
-
   if (GTK_OBJECT_CLASS(parent_class)->destroy)
     (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
 }
@@ -939,14 +1021,21 @@ typedef struct _DoOnSelectionArgs
 
 
 static void
-do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
-		       GtkTreeIter *iter, DoOnSelectionArgs *args)
+do_on_selection_helper(gpointer list_data, gpointer user_data)
 {
-  gpointer data = NULL;
+  GtkTreePath *path = (GtkTreePath *)list_data;
+  DoOnSelectionArgs *args = (DoOnSelectionArgs *)user_data;
+  GtkTreeModel *model = args->view->model;
   int what_to_do = args->what_to_do;
+  gpointer data = NULL;
+  GtkTreeIter iter;
 
-  gtk_tree_model_get(model, iter, BOARDLIST_DATA, &data, -1);
-  g_return_if_fail(data != NULL);
+  if (!gtk_tree_model_get_iter(model, &iter, path))
+    goto done;
+
+  gtk_tree_model_get(model, &iter, BOARDLIST_DATA, &data, -1);
+  if (data == NULL)
+    goto done;	/* ͭʤǰΤ */
 
   switch (what_to_do)
     {
@@ -985,7 +1074,7 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
 			boardlist_view_signals[TOGGLE_HIDE_CATEGORY_SIGNAL],
 			0,
 			category);
-	  gtk_tree_store_set(GTK_TREE_STORE(model), iter,
+	  gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
 			     BOARDLIST_ROW_STATE,
 			     encode_row_state(category->hidden,
 					      category->killed),
@@ -1002,7 +1091,7 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
 			boardlist_view_signals[TOGGLE_HIDE_BOARD_SIGNAL],
 			0,
 			board);
-	  gtk_tree_store_set(GTK_TREE_STORE(model), iter,
+	  gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
 			     BOARDLIST_ROW_STATE,
 			     encode_row_state(board->hidden, board->killed),
 			     -1);
@@ -1021,7 +1110,7 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
 			boardlist_view_signals[KILL_CATEGORY_SIGNAL],
 			0,
 			category);
-	  gtk_tree_store_set(GTK_TREE_STORE(model), iter,
+	  gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
 			     BOARDLIST_ROW_STATE,
 			     encode_row_state(category->hidden,
 					      category->killed),
@@ -1032,18 +1121,33 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
 	}
       else if (OCHUSHA_IS_BULLETIN_BOARD(data))
 	{
+	  GtkTreePath *category_path = gtk_tree_path_copy(path);
 	  OchushaBulletinBoard *board = OCHUSHA_BULLETIN_BOARD(data);
+	  OchushaBoardCategory *category = NULL;
+
+	  if (gtk_tree_path_up(category_path))
+	    {
+	      GtkTreeIter category_iter;
+	      if (gtk_tree_model_get_iter(model, &category_iter,
+					  category_path))
+		{
+		  gtk_tree_model_get(model, &category_iter,
+				     BOARDLIST_DATA, &category, -1);
+		}
+	    }
 	  g_signal_emit(G_OBJECT(args->view),
 			boardlist_view_signals[KILL_BOARD_SIGNAL],
 			0,
+			category,
 			board);
-	  gtk_tree_store_set(GTK_TREE_STORE(model), iter,
+	  gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
 			     BOARDLIST_ROW_STATE,
 			     encode_row_state(board->hidden, board->killed),
 			     -1);
 	  args->hidden_boards
 	    = g_slist_append(args->hidden_boards,
 			     gtk_tree_row_reference_new(model, path));
+	  gtk_tree_path_free(category_path);
 	}
       break;
 
@@ -1055,6 +1159,9 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
 		      data);
       break;
     }
+
+ done:
+  gtk_tree_path_free(path);
 }
 
 
@@ -1081,6 +1188,7 @@ remove_row(gpointer list_data, gpointer user_data)
 static void
 boardlist_view_do_on_selection(BoardlistView *view, int what_to_do)
 {
+  GList *rows;
   DoOnSelectionArgs args = { view, what_to_do, NULL, NULL };
 #if DEBUG_WIDGET_MOST
   fprintf(stderr, "boardlist_view_do_on_selection: view=%p what_to_do=%d\n",
@@ -1088,9 +1196,9 @@ boardlist_view_do_on_selection(BoardlistView *view, int what_to_do)
 #endif
   MAIN_THREAD_EVENT_CALLBACK();
 
-  gtk_tree_selection_selected_foreach(view->selection,
-			(GtkTreeSelectionForeachFunc)do_on_selection_helper,
-			&args);
+  rows = gtk_tree_selection_get_selected_rows(view->selection, NULL);
+  g_list_foreach(rows, do_on_selection_helper, &args);
+  g_list_free(rows);
 
   if (args.hidden_boards != NULL)
     {
@@ -1719,3 +1827,165 @@ boardlist_view_open(BoardlistView *view, GSList *category_list,
 }
 
 
+void
+boardlist_view_add_category(BoardlistView *view,
+			    OchushaBoardCategory *category,
+			    OchushaBoardCategory *prev_category)
+{
+  CategoryNodeInfo *info;
+  GtkTreeIter iter;
+  GtkTreeIter prev_iter;
+  GtkTreeIter *prev_iter_p = NULL;
+  GtkTreePath *path;
+
+  g_return_if_fail(IS_BOARDLIST_VIEW(view));
+  g_return_if_fail(OCHUSHA_IS_BOARD_CATEGORY(category));
+
+  info = ensure_node_info(G_OBJECT(category));
+  g_return_if_fail(info != NULL);
+  if (OCHUSHA_IS_BOARD_CATEGORY(prev_category))
+    {
+      CategoryNodeInfo *prev_info = ensure_node_info(G_OBJECT(prev_category));
+      path = gtk_tree_row_reference_get_path(prev_info->reference);
+      if (path != NULL)
+	{
+	  if (gtk_tree_model_get_iter(view->model, &prev_iter, path))
+	    prev_iter_p = &prev_iter;
+	  gtk_tree_path_free(path);
+	}
+    }
+
+  gtk_tree_store_insert_after(GTK_TREE_STORE(view->model), &iter,
+			      NULL, prev_iter_p);
+  gtk_tree_store_set(GTK_TREE_STORE(view->model), &iter,
+		     BOARDLIST_LABEL, category->name,
+		     BOARDLIST_ROW_TYPE, BOARDLIST_CATEGORY_ROW,
+		     BOARDLIST_ROW_STATE, encode_row_state(category->hidden,
+							   category->killed),
+		     BOARDLIST_DATA, category,
+		     -1);
+  path = gtk_tree_model_get_path(view->model, &iter);
+  if (info->reference != NULL)
+    gtk_tree_row_reference_free(info->reference);
+  info->reference = gtk_tree_row_reference_new(view->model, path);
+  gtk_tree_path_free(path);
+}
+
+
+void
+boardlist_view_remove_category(BoardlistView *view,
+			       OchushaBoardCategory *category)
+{
+  CategoryNodeInfo *info;
+
+  g_return_if_fail(IS_BOARDLIST_VIEW(view));
+  g_return_if_fail(OCHUSHA_IS_BOARD_CATEGORY(category));
+
+  info = ensure_node_info(G_OBJECT(category));
+  g_return_if_fail(info->reference != NULL);
+  remove_row(info->reference, view->model);
+  info->reference = NULL;
+}
+
+
+void
+boardlist_view_add_board(BoardlistView *view, OchushaBulletinBoard *board,
+			 OchushaBoardCategory *category,
+			 OchushaBulletinBoard *prev_board)
+{
+  CategoryNodeInfo *info;
+  GtkTreeIter category_iter;
+  GtkTreePath *path;
+
+  g_return_if_fail(IS_BOARDLIST_VIEW(view));
+  g_return_if_fail(OCHUSHA_IS_BOARD_CATEGORY(category));
+  g_return_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board));
+
+  info = ensure_node_info(G_OBJECT(category));
+  g_return_if_fail(info != NULL || info->reference != NULL);
+
+  path = gtk_tree_row_reference_get_path(info->reference);
+  g_return_if_fail(path != NULL);
+
+  if (gtk_tree_model_get_iter(view->model, &category_iter, path))
+    {
+      GtkTreeIter prev_iter;
+      GtkTreeIter *prev_iter_p = NULL;
+      GtkTreeIter iter;
+
+      if (OCHUSHA_IS_BULLETIN_BOARD(prev_board))
+	{
+	  /* prev_iter_pν */
+	  if (gtk_tree_model_iter_children(view->model, &prev_iter,
+					   &category_iter))
+	    {
+	      do
+		{
+		  OchushaBulletinBoard *data;
+		  gtk_tree_model_get(view->model, &prev_iter,
+				     BOARDLIST_DATA, &data, -1);
+		  if (data == prev_board)
+		    {
+		      prev_iter_p = &prev_iter;
+		      break;
+		    }
+		} while (gtk_tree_model_iter_next(view->model, &prev_iter));
+	    }
+	}
+
+      gtk_tree_store_insert_after(GTK_TREE_STORE(view->model), &iter,
+				  &category_iter, prev_iter_p);
+      gtk_tree_store_set(GTK_TREE_STORE(view->model), &iter,
+			 BOARDLIST_LABEL, board->name,
+			 BOARDLIST_ROW_TYPE, BOARDLIST_BOARD_ROW,
+			 BOARDLIST_ROW_STATE, encode_row_state(board->hidden,
+							       board->killed),
+			 BOARDLIST_DATA, board,
+			 -1);
+    }
+
+  gtk_tree_path_free(path);
+}
+
+
+void
+boardlist_view_remove_board(BoardlistView *view,
+			    OchushaBoardCategory *category,
+			    OchushaBulletinBoard *board)
+{
+  CategoryNodeInfo *info;
+  GtkTreeIter category_iter;
+  GtkTreePath *path;
+
+  g_return_if_fail(IS_BOARDLIST_VIEW(view));
+  g_return_if_fail(OCHUSHA_IS_BOARD_CATEGORY(category));
+  g_return_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board));
+
+  info = ensure_node_info(G_OBJECT(category));
+  g_return_if_fail(info != NULL || info->reference != NULL);
+
+  path = gtk_tree_row_reference_get_path(info->reference);
+  g_return_if_fail(path != NULL);
+
+  if (gtk_tree_model_get_iter(view->model, &category_iter, path))
+    {
+      GtkTreeIter iter;
+      if (gtk_tree_model_iter_children(view->model, &iter, &category_iter))
+	{
+	  do
+	    {
+	      OchushaBulletinBoard *data;
+	      gtk_tree_model_get(view->model, &iter,
+				 BOARDLIST_DATA, &data, -1);
+	      if (data == board)
+		{
+		  gtk_tree_store_remove(GTK_TREE_STORE(view->model), &iter);
+		  break;
+		}
+	    } while (gtk_tree_model_iter_next(view->model, &iter));
+	}
+    }
+
+  gtk_tree_path_free(path);
+}
+
