@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.h,v 1.11 2004/01/01 17:16:16 fuyu Exp $
+ * $Id: boardlist_view.h,v 1.15 2004/01/10 23:37:50 fuyu Exp $
  */
 
 #ifndef _BOARDLIST_VIEW_H_
@@ -58,8 +58,6 @@ struct _BoardlistView
   GtkTreeSelection *selection;
 
   GtkTreePath *last_event_path;
-  GSList *last_drag_event_data;
-  guint last_event_time;
 
   gboolean hide_hidden_items;
   gboolean default_open_in_tab;
@@ -70,35 +68,35 @@ struct _BoardlistViewClass
 {
   GtkTreeViewClass parent_class;
 
-  void (*open_threadlist_view)(BoardlistView *boardlist_view,
+  void (*open_threadlist_view)(BoardlistView *view,
 			       OchushaBulletinBoard *board,
 			       gboolean in_tab, gboolean with_browser);
 
-  void (*select_category)(BoardlistView *boardlist_view,
-			  OchushaBoardCategory *category);
-  void (*category_expanded)(BoardlistView *boardlist_view,
+  void (*select_category)(BoardlistView *view, OchushaBoardCategory *category);
+  void (*category_expanded)(BoardlistView *view,
 			    OchushaBoardCategory *category);
-  void (*category_collapsed)(BoardlistView *boardlist_view,
+  void (*category_collapsed)(BoardlistView *view,
 			     OchushaBoardCategory *category);
 
-  void (*toggle_hide_category)(BoardlistView *boardlist_view,
+  void (*toggle_hide_category)(BoardlistView *view,
 			       OchushaBoardCategory *category);
-  void (*toggle_hide_board)(BoardlistView *boardlist_view,
+  void (*toggle_hide_board)(BoardlistView *view,
 			    OchushaBulletinBoard *board);
-  void (*kill_category)(BoardlistView *boardlist_view,
-			OchushaBoardCategory *category);
-  void (*kill_board)(BoardlistView *boardlist_view,
+  void (*kill_category)(BoardlistView *view, OchushaBoardCategory *category);
+  void (*kill_board)(BoardlistView *view, OchushaBoardCategory *category,
 		     OchushaBulletinBoard *board);
-  void (*copy_board_url)(BoardlistView *boardlist_view,
-			 OchushaBulletinBoard *board);
+  void (*copy_board_url)(BoardlistView *view, OchushaBulletinBoard *board);
 
-  gboolean (*advance_view)(BoardlistView *boardlist_view);
-  gboolean (*back_view)(BoardlistView *boardlist_view);
+  gboolean (*advance_view)(BoardlistView *view);
+  gboolean (*back_view)(BoardlistView *view);
 
-  void (*do_on_selection)(BoardlistView *boardlist_view,
-			  int what_to_do);
-  void (*do_on_item_at_cursor)(BoardlistView *boardlist_view,
-			       int what_to_do);
+  void (*do_on_selection)(BoardlistView *view, int what_to_do);
+  void (*do_on_item_at_cursor)(BoardlistView *view, int what_to_do);
+
+  void (*uri_dropped)(BoardlistView *view,
+		      OchushaBoardCategory *category,
+		      OchushaBulletinBoard *board, const char *uri,
+		      GtkTreeViewDropPosition pos);
 };
 
 
@@ -107,6 +105,19 @@ GtkWidget *boardlist_view_new(void);
 void boardlist_view_open(BoardlistView *view, GSList *category_list,
 			 gboolean really_hide_hidden_items);
 
+void boardlist_view_add_category(BoardlistView *view,
+				 OchushaBoardCategory *category,
+				 OchushaBoardCategory *prev_category);
+void boardlist_view_remove_category(BoardlistView *view,
+				    OchushaBoardCategory *category);
+void boardlist_view_add_board(BoardlistView *view,
+			      OchushaBulletinBoard *board,
+			      OchushaBoardCategory *category,
+			      OchushaBulletinBoard *prev_board);
+void boardlist_view_remove_board(BoardlistView *view,
+				 OchushaBoardCategory *category,
+				 OchushaBulletinBoard *board);
+
 gboolean boardlist_view_get_default_open_in_tab(BoardlistView *view);
 void boardlist_view_set_default_open_in_tab(BoardlistView *view,
 					    gboolean default_open_in_tab);
