@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.94 2004/01/04 06:44:49 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.111 2004/01/11 03:03:58 fuyu Exp $
  */
 
 #include "config.h"
@@ -40,11 +40,13 @@
 #include "bulletin_board_ui.h"
 #include "icon_label.h"
 #include "text_search_window.h"
+#include "thread_proxy.h"
 #include "threadlist_filter.h"
 #include "threadlist_view.h"
 #include "paned_notebook.h"
 #include "response_editor.h"
 #include "ugly_gtk2utils.h"
+#include "virtual_board.h"
 
 #include "worker.h"
 
@@ -85,8 +87,12 @@ static void toggle_hide_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			   OchushaApplication *application);
 static void reset_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			    OchushaApplication *application);
+static void kill_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
+			   VirtualBoard *board);
 static void copy_thread_url_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			       OchushaApplication *application);
+static void bookmark_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
+			       OchushaApplication *application);
 static void mark_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			   gboolean do_mark, OchushaApplication *application);
 static void thread_mouse_over_cb(ThreadlistView *view,
@@ -131,6 +137,10 @@ static void go_to_bottom_button_cb(GtkWidget *widget,
 				   OchushaApplication *application);
 static void go_to_top_button_cb(GtkWidget *widget,
 				OchushaApplication *application);
+static void go_forward_button_cb(GtkWidget *widget,
+				 OchushaApplication *application);
+static void go_back_button_cb(GtkWidget *widget,
+			      OchushaApplication *application);
 static GtkWidget *create_write_dialog(OchushaApplication *application,
 				      OchushaBBSThread *thread);
 
@@ -191,10 +201,11 @@ static TSCellHandle *list_entry_decorate_func;
 static TSCellHandle *calculate_weight_func;
 
 static const char *default_init_scm =
-  "(define (list-entry-decorate-default new-responses filter-enabled filter-ok new-thread favorite hidden) (cond (hidden (cons list-entry-fg-hidden list-entry-bg-hidden)) (new-thread (cons list-entry-fg-strong list-entry-bg-strong)) (filter-ok (cons (if new-responses list-entry-fg-emph list-entry-fg-normal) (if filter-enabled list-entry-bg-normal list-entry-bg-emph))) nil))"
-  "(define (list-entry-decorate-default-old new-responses filter-enabled filter-ok new-thread favorite hidden) (cons (if hidden list-entry-fg-hidden (if (and new-responses favorite) list-entry-fg-emph list-entry-fg-normal)) (if new-thread list-entry-bg-strong (if (and new-responses filter-ok) list-entry-bg-emph list-entry-bg-normal))))"
+  "(define (list-entry-decorate-default new-responses filter-enabled filter-ok new-thread marked hidden) (cond (hidden (cons list-entry-fg-hidden list-entry-bg-hidden)) (new-thread (cons list-entry-fg-strong list-entry-bg-strong)) (filter-ok (cons (if new-responses list-entry-fg-emph list-entry-fg-normal) (if filter-enabled list-entry-bg-normal list-entry-bg-emph))) nil))"
+  "(define (list-entry-decorate-default-old new-responses filter-enabled filter-ok new-thread marked hidden) (cons (if hidden list-entry-fg-hidden (if (and new-responses marked) list-entry-fg-emph list-entry-fg-normal)) (if new-thread list-entry-bg-strong (if (and new-responses filter-ok) list-entry-bg-emph list-entry-bg-normal))))"
   "(define list-entry-decorate list-entry-decorate-default)"
-  "(define (calculate-weight-default rank n_res n_got n_read dead new favorite hidden) (+ (if hidden 100000 0) (if favorite 0 20000) (if (> n_got 0) 0 10000) (if new 0 5000) (if dead 10000 0)))"
+  "(define (calculate-weight-default rank n-res n-got n-read dead new marked hidden) (+ (if hidden 1000 0) (if marked 0 100) (if (> n-got 0) (if (> n-res n-got) 0 200) 400) (if new 0 100) (if dead 500 0)))"
+  "(define (calculate-weight-default-old rank n_res n_got n_read dead new marked hidden) (+ (if hidden 1000 0) (if marked 0 200) (if (> n_got 0) 0 100) (if new 0 50) (if dead 100 0)))"
   "(define calculate-weight calculate-weight-default)";
 
 
@@ -358,12 +369,13 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
   info->last_mail = ochusha_utils_get_attribute_string(hash_table,
 						       "last_mail");
   if ((info->view_flags & BBS_THREAD_HIDDEN) != 0)
-    thread->number_of_responses_read = 0;
+    ochusha_bbs_thread_set_number_of_responses_read(thread, 0);
 
   info->bookmark_response_number
     = ochusha_utils_get_attribute_int(hash_table, "bookmark_response_number");
   if (info->bookmark_response_number == 0)
-    info->bookmark_response_number = thread->number_of_responses_read;
+    info->bookmark_response_number
+      = ochusha_bbs_thread_get_number_of_responses_read(thread);
 
   info->thread_local_a_bone
     = ochusha_utils_get_attribute_boolean(hash_table, "thread_local_a_bone");
@@ -449,12 +461,6 @@ threadlist_write_thread_element_cb(OchushaBulletinBoard *board,
 				  bookmark_response_number);
     }
 
-  if ((info->view_flags & BBS_THREAD_HIDDEN) != 0)
-    {
-      thread->number_of_responses_read = 0;
-      ochusha_bbs_thread_remove_cache(thread, &application->config);
-    }
-
   if (info->thread_local_a_bone)
     {
       OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(threadlist_xml, info,
@@ -633,20 +639,43 @@ title_match_func(const gchar *title, ThreadlistSearchQueryData *search_data)
 
 static void
 customize_threadlist_view_contents(OchushaApplication *application,
-				   ThreadlistView *view)
+				   ThreadlistView *view,
+				   OchushaBulletinBoard *board)
 {
-  const char *column = application->threadlist_view_contents;
-  if (column == NULL)
+  char tmp_column[64];
+  const char *column;
+  gboolean is_virtual_board = IS_VIRTUAL_BOARD(board);
+
+  if (application->threadlist_view_contents == NULL)
     return;
 
+  if (is_virtual_board)
+    {
+      if (strchr(application->threadlist_view_contents, 'B') == NULL)
+	{
+	  snprintf(tmp_column, 64, "B%s",
+		   application->threadlist_view_contents);
+	  column = tmp_column;
+	}
+      else
+	column = application->threadlist_view_contents;
+    }
+  else
+    column = application->threadlist_view_contents;
+
   g_return_if_fail(strlen(column) > 0);
 
   while (*column != '\0')
     {
       switch (*column)
 	{
+	case 'B':
+	  threadlist_view_append_board(view);
+	  break;
+
 	case 'R':
-	  threadlist_view_append_rank(view);
+	  if (!is_virtual_board)
+	    threadlist_view_append_rank(view);
 	  break;
 
 	case 'M':
@@ -682,7 +711,8 @@ customize_threadlist_view_contents(OchushaApplication *application,
 	  break;
 
 	case 'D':
-	  threadlist_view_append_rank_difference(view);
+	  if (!is_virtual_board)
+	    threadlist_view_append_rank_difference(view);
 	  break;
 
 	case 'L':
@@ -762,13 +792,18 @@ open_bulletin_board(OchushaApplication *application,
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
 				   GTK_POLICY_AUTOMATIC,
 				   GTK_POLICY_AUTOMATIC);
+#if 0	/* themeǤˤޤ٤ */
     gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window),
 					GTK_SHADOW_ETCHED_IN);
+#endif
     view = threadlist_view_new();
     g_object_set_qdata(G_OBJECT(view), application_id, application);
-    customize_threadlist_view_contents(application, THREADLIST_VIEW(view));
+    customize_threadlist_view_contents(application, THREADLIST_VIEW(view),
+				       board);
     threadlist_view_set_default_open_in_tab(THREADLIST_VIEW(view),
 					    application->default_open_in_tab);
+    threadlist_view_set_remove_hidden_thread(THREADLIST_VIEW(view),
+					     application->hide_hidden_threads);
 
     job = G_NEW0(WorkerJob, 1);
     job_args = G_NEW0(ThreadlistJobArgs, 1);
@@ -799,7 +834,7 @@ open_bulletin_board(OchushaApplication *application,
 
     gtk_toolbar_prepend_space(toolbar);
     gtk_toolbar_insert_stock(toolbar, OCHUSHA_STOCK_WRITE_RESPONSE,
-			     _("Write a response to the current thread"),
+			     _("Write a Response to the Current Thread"),
 			     "write_response_to_current_thread",
 			     GTK_SIGNAL_FUNC(write_response_button_cb),
 			     application,
@@ -807,44 +842,64 @@ open_bulletin_board(OchushaApplication *application,
 
     gtk_toolbar_prepend_space(toolbar);
     gtk_toolbar_insert_stock(toolbar, GTK_STOCK_SELECT_FONT,
-			     _("Select font to render threads"),
+			     _("Select Font to Render Threads"),
 			     "select_font_to_render_threads",
 			     GTK_SIGNAL_FUNC(select_font_button_cb),
 			     application,
 			     0);
 
     gtk_toolbar_prepend_space(toolbar);
-    gtk_toolbar_insert_stock(toolbar, GTK_STOCK_REFRESH,
-			     _("Refresh current thread"),
-			     "refresh_current_thread",
-			     GTK_SIGNAL_FUNC(refresh_thread_button_cb),
-			     application,
-			     0);
     gtk_toolbar_insert_stock(toolbar, GTK_STOCK_FIND,
-			     _("Start text search on current thread"),
+			     _("Start Text Search on the Current Thread"),
 			     "start_text_search_on_thread",
 			     GTK_SIGNAL_FUNC(start_thread_search_button_cb),
 			     application,
 			     0);
     gtk_toolbar_insert_stock(toolbar, GTK_STOCK_JUMP_TO,
-			     _("Jump to bookmark"),
+			     _("Jump to Bookmark"),
 			     "jump_to_bookmark",
 			     GTK_SIGNAL_FUNC(jump_to_bookmark_button_cb),
 			     application,
 			     0);
     gtk_toolbar_insert_stock(toolbar, GTK_STOCK_GOTO_BOTTOM,
-			     _("Go to bottom of the response view"),
+			     _("Go to Bottom of the Response View"),
 			     "go_to_bottom_of_response",
 			     GTK_SIGNAL_FUNC(go_to_bottom_button_cb),
 			     application,
 			     0);
     gtk_toolbar_insert_stock(toolbar, GTK_STOCK_GOTO_TOP,
-			     _("Go to top of the response view"),
+			     _("Go to Top of the Response View"),
 			     "go_to_top_of_response",
 			     GTK_SIGNAL_FUNC(go_to_top_button_cb),
 			     application,
 			     0);
 
+#if 0
+    gtk_toolbar_prepend_space(toolbar);
+#endif
+    gtk_toolbar_insert_stock(toolbar, GTK_STOCK_REFRESH,
+			     _("Refresh the Current Thread"),
+			     "refresh_current_thread",
+			     GTK_SIGNAL_FUNC(refresh_thread_button_cb),
+			     application,
+			     0);
+
+#if 0
+    gtk_toolbar_prepend_space(toolbar);
+#endif
+    gtk_toolbar_insert_stock(toolbar, GTK_STOCK_GO_FORWARD,
+			     _("Go to Next View Position"),
+			     "go_to_next_position",
+			     GTK_SIGNAL_FUNC(go_forward_button_cb),
+			     application,
+			     0);
+    gtk_toolbar_insert_stock(toolbar, GTK_STOCK_GO_BACK,
+			     _("Go to Previous View Position"),
+			     "go_to_previous_position",
+			     GTK_SIGNAL_FUNC(go_back_button_cb),
+			     application,
+			     0);
+
     gtk_widget_show(paned_notebook);
 
     g_signal_connect(G_OBJECT(view), "advance_view",
@@ -861,8 +916,18 @@ open_bulletin_board(OchushaApplication *application,
 		     G_CALLBACK(toggle_hide_cb), application);
     g_signal_connect(G_OBJECT(view), "reset_thread",
 		     G_CALLBACK(reset_thread_cb), application);
+    if (IS_VIRTUAL_BOARD(board))
+      {
+	threadlist_view_set_editable(THREADLIST_VIEW(view), TRUE);
+	g_signal_connect(G_OBJECT(view), "kill_thread",
+			 G_CALLBACK(kill_thread_cb), board);
+      }
+    else
+      threadlist_view_set_editable(THREADLIST_VIEW(view), FALSE);
     g_signal_connect(G_OBJECT(view), "copy_thread_url",
 		     G_CALLBACK(copy_thread_url_cb), application);
+    g_signal_connect(G_OBJECT(view), "bookmark_thread",
+		     G_CALLBACK(bookmark_thread_cb), application);
     g_signal_connect(G_OBJECT(view), "mark_thread",
 		     G_CALLBACK(mark_thread_cb), application);
     g_signal_connect(G_OBJECT(view), "thread_mouse_over",
@@ -899,7 +964,7 @@ open_bulletin_board(OchushaApplication *application,
     info->is_busy = TRUE;
     info->threadlist_source_buffer
       = ochusha_bulletin_board_get_threadlist_source(board,
-				application->broker,
+				application->broker, NULL,
 				OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH);
     if (info->threadlist_source_buffer != NULL)
       setup_for_tab_label_animation(info->threadlist_source_buffer,
@@ -1069,7 +1134,7 @@ refresh_threadlist_view(OchushaApplication *application,
 
     info->threadlist_source_buffer
       = ochusha_bulletin_board_get_threadlist_source(board,
-				application->broker,
+				application->broker, old_buffer,
 				OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH);
     if (info->threadlist_source_buffer == NULL)
       {
@@ -1078,6 +1143,9 @@ refresh_threadlist_view(OchushaApplication *application,
 	return;
       }
 
+    if (info->threadlist_source_buffer == old_buffer)
+      old_buffer = NULL;
+
     setup_for_tab_label_animation(info->threadlist_source_buffer, tab_label);
 
     job = G_NEW0(WorkerJob, 1);
@@ -1100,8 +1168,8 @@ refresh_threadlist_view(OchushaApplication *application,
     OCHU_OBJECT_REF(G_OBJECT(threadlist_view));
 
     commit_job(job);
-
-    OCHU_OBJECT_UNREF(G_OBJECT(old_buffer));
+    if (old_buffer != NULL)
+      OCHU_OBJECT_UNREF(G_OBJECT(old_buffer));
   }
   THREAD_LIST_UNLOCK;
 }
@@ -1249,7 +1317,7 @@ open_kako_thread_view(WorkerThread *employee, gpointer job_args)
     if (thread == NULL)
       thread = ochusha_bulletin_board_lookup_kako_thread_by_url(board,
 							application->broker,
-							args->url);
+							NULL, args->url);
 
 #if DEBUG_GUI_MOST
     fprintf(stderr, "Kako Thread URL: \"%s\"\n", args->url);
@@ -1292,27 +1360,42 @@ typedef struct _EachThreadArgs
 
 
 static int
-calculate_weight(OchushaBBSThread *thread, BBSThreadGUIInfo *info, int rank)
+calculate_weight(OchushaBBSThread *thread, BBSThreadGUIInfo *info, int rank,
+		 gboolean use_builtin)
 {
   int weight = 0;
-  if (engine != NULL && calculate_weight_func != NULL)
+  if (!use_builtin && engine != NULL && calculate_weight_func != NULL)
     {
       TSCellHandle *result
 	= ts_engine_evalf0(engine, calculate_weight_func,
 			   "%d%d%d%d%C%C%C%C",
 			   rank,
-			   thread->number_of_responses_on_server,
-			   thread->number_of_responses_read,
+			   ochusha_bbs_thread_get_number_of_responses_on_server(thread),
+			   ochusha_bbs_thread_get_number_of_responses_read(thread),
 			   info->bookmark_response_number,
-			   TS_BOOL(thread->flags & (OCHUSHA_BBS_THREAD_DAT_DROPPED | OCHUSHA_BBS_THREAD_KAKO)),
+			   TS_BOOL(ochusha_bbs_thread_get_flags(thread) & (OCHUSHA_BBS_THREAD_DAT_DROPPED | OCHUSHA_BBS_THREAD_KAKO)),
 			   TS_BOOL(info->view_flags & BBS_THREAD_NEW),
-			   TS_BOOL(info->view_flags & BBS_THREAD_FAVORITE),
+			   TS_BOOL(info->view_flags & BBS_THREAD_MARKED),
 			   TS_BOOL(info->view_flags & BBS_THREAD_HIDDEN));
 
       if (ts_cell_handle_is_long(result))
 	weight = (int)ts_cell_handle_get_long_value(result);
       g_object_unref(G_OBJECT(result));
     }
+  else
+    {
+      weight
+	= ((info->view_flags & BBS_THREAD_HIDDEN) ? 1000 : 0)
+	+ ((info->view_flags & BBS_THREAD_MARKED) ? 0 : 100)
+	+ ((ochusha_bbs_thread_get_number_of_responses_read(thread) > 0)
+	   ? (ochusha_bbs_thread_get_number_of_responses_read(thread)
+	      < ochusha_bbs_thread_get_number_of_responses_on_server(thread)
+	      ? 0 : 200 ) : 400)
+	+ ((info->view_flags & BBS_THREAD_NEW) ? 0 : 100)
+	+ ((ochusha_bbs_thread_get_flags(thread)
+	    & (OCHUSHA_BBS_THREAD_DAT_DROPPED | OCHUSHA_BBS_THREAD_KAKO))
+	   ? 500 : 0);
+    }
   return weight;
 }
 
@@ -1355,12 +1438,13 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
       OCHU_THREADS_ENTER();
 
       threadlist_view_append_thread(view, thread, info->num_shown,
-				    (MAX(thread->number_of_responses_on_server,
-					 thread->number_of_responses_read)
+				    (MAX(ochusha_bbs_thread_get_number_of_responses_on_server(thread),
+					 ochusha_bbs_thread_get_number_of_responses_read(thread))
 				     - info->bookmark_response_number),
 				    info->view_rank, args->number_of_threads,
 				    calculate_weight(thread, info,
-						     args->number_of_threads));
+						     args->number_of_threads,
+						     args->application->use_builtin_calculate_weight));
       update_threadlist_entry_style(args->application, view, thread);
 
       OCHU_THREADS_LEAVE();
@@ -1403,7 +1487,6 @@ show_threadlist(WorkerThread *employee, gpointer args)
   }
 #endif
 
-  /* ˽λʤäˤ٤ */
   ochusha_bulletin_board_read_threadlist_xml(board, &application->config,
 					     application->session_subdir,
 					     info->recover_mode);
@@ -1444,6 +1527,8 @@ show_threadlist(WorkerThread *employee, gpointer args)
 	= gtk_statusbar_push(application->statusbar, threadlist_netstat_id,
 			     message);
     }
+
+  threadlist_view_close(view);
   if (!threadlist_view_open(view))
     {
       OCHU_THREADS_LEAVE();
@@ -1469,22 +1554,11 @@ show_threadlist(WorkerThread *employee, gpointer args)
       fprintf(stderr, "Broken thread list (subject.txt) for %s\n", name);
       G_FREE(name);
 #endif
-
-      OCHU_THREADS_ENTER();
-
-      threadlist_view_close(view, NULL);
-
-      OCHU_THREADS_LEAVE();
+      threadlist_view_close(view);
 
       goto finish_job;
     }
   
-  OCHU_THREADS_ENTER();
-
-  threadlist_view_close(view, cb_args.last_thread);
-
-  OCHU_THREADS_LEAVE();
-
  finish_job:
 
   OCHU_THREADS_ENTER();
@@ -1567,13 +1641,13 @@ update_threadlist_entry_style(OchushaApplication *application,
     };
 
   gboolean exist_new_responses
-    = (thread->number_of_responses_on_server
-       > thread->number_of_responses_read);
+    = (ochusha_bbs_thread_get_number_of_responses_on_server(thread)
+       > ochusha_bbs_thread_get_number_of_responses_read(thread));
 
   /* flags */
   if (info->view_flags & BBS_THREAD_HIDDEN)
     visual_parameter.flags |= THREADLIST_ENTRY_FLAG_STRIKE;
-  if (info->view_flags & BBS_THREAD_FAVORITE)
+  if (info->view_flags & BBS_THREAD_MARKED)
     visual_parameter.flags |= THREADLIST_ENTRY_FLAG_MARK;
 
   if (engine != NULL && list_entry_decorate_func != NULL)
@@ -1588,7 +1662,7 @@ update_threadlist_entry_style(OchushaApplication *application,
 			   TS_BOOL(board_info->enable_filter),
 			   TS_BOOL(filter_result),
 			   TS_BOOL(info->view_flags & BBS_THREAD_NEW),
-			   TS_BOOL(info->view_flags & BBS_THREAD_FAVORITE),
+			   TS_BOOL(info->view_flags & BBS_THREAD_MARKED),
 			   TS_BOOL(info->view_flags & BBS_THREAD_HIDDEN));
 
       if (ts_cell_handle_is_pair(result))
@@ -1628,12 +1702,21 @@ update_threadlist_entry_style(OchushaApplication *application,
 	}
     }
 
+#if 0
   threadlist_view_update_thread(view, thread, info->num_shown,
-				(MAX(thread->number_of_responses_on_server,
-				     thread->number_of_responses_read)
+				(MAX(ochusha_bbs_thread_get_number_of_responses_on_server(thread),
+				     ochusha_bbs_thread_get_number_of_responses_read(thread))
 				 - info->bookmark_response_number),
 				calculate_weight(thread, info,
-						 info->view_rank));
+						 info->view_rank,
+						 application->use_builtin_calculate_weight));
+#else
+  threadlist_view_update_thread(view, thread, info->num_shown,
+				(MAX(ochusha_bbs_thread_get_number_of_responses_on_server(thread),
+				     ochusha_bbs_thread_get_number_of_responses_read(thread))
+				 - info->bookmark_response_number),
+				-1);
+#endif
   threadlist_view_update_thread_visual(view, thread, &visual_parameter);
 }
 
@@ -1695,7 +1778,7 @@ toggle_mark_cb(ThreadlistView *view, OchushaBBSThread *thread,
 	       OchushaApplication *application)
 {
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
-  info->view_flags ^= BBS_THREAD_FAVORITE;
+  info->view_flags ^= BBS_THREAD_MARKED;
   update_threadlist_entry_style(application, view, thread);
 }
 
@@ -1706,8 +1789,6 @@ toggle_hide_cb(ThreadlistView *view, OchushaBBSThread *thread,
 {
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   info->view_flags ^= BBS_THREAD_HIDDEN;
-  if ((info->view_flags & BBS_THREAD_HIDDEN) != 0)
-    ochusha_bbs_thread_remove_cache(thread, &application->config);
   update_threadlist_entry_style(application, view, thread);
 }
 
@@ -1724,6 +1805,14 @@ reset_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
 }
 
 
+static void
+kill_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
+	       VirtualBoard *board)
+{
+  virtual_board_remove_thread(board, thread);
+}
+
+
 static void
 copy_thread_url_cb(ThreadlistView *view, OchushaBBSThread *thread,
 		   OchushaApplication *application)
@@ -1732,15 +1821,65 @@ copy_thread_url_cb(ThreadlistView *view, OchushaBBSThread *thread,
 }
 
 
+void
+bookmark_thread(OchushaBulletinBoard *board, OchushaBBSThread *thread,
+		OchushaApplication *application)
+{
+  GtkWidget *widget;
+  const char *id = ochusha_bbs_thread_get_url(thread);
+  OchushaBBSThread *proxy
+    = ochusha_bulletin_board_lookup_bbs_thread_by_id(board, id);
+  if (proxy != NULL)
+    return;	/* bookmark */
+
+  g_return_if_fail(IS_VIRTUAL_BOARD(board));
+
+  proxy = ochusha_bulletin_board_bbs_thread_new(board, id, thread->title);
+  thread_proxy_set_real_thread(THREAD_PROXY(proxy), thread);
+  board->thread_list = g_slist_append(board->thread_list, proxy);
+
+  widget = paned_notebook_get_item_view(PANED_NOTEBOOK(application->contents_window), board);
+  if (widget != NULL)
+    {
+      /* widgetPanedNotebook */
+      widget = PANED_NOTEBOOK(widget)->selector;
+      if (GTK_IS_SCROLLED_WINDOW(widget))
+	{
+	  ThreadlistView *favorites_view
+	    = THREADLIST_VIEW(gtk_bin_get_child(GTK_BIN(widget)));
+	  BBSThreadGUIInfo *info = ensure_bbs_thread_info(proxy);
+	  threadlist_view_append_thread(favorites_view,
+					proxy, info->num_shown,
+					(MAX(ochusha_bbs_thread_get_number_of_responses_on_server(proxy),
+					 ochusha_bbs_thread_get_number_of_responses_read(proxy))
+					 - info->bookmark_response_number),
+					info->view_rank, 1,
+					calculate_weight(proxy, info,
+							 1,
+							 application->use_builtin_calculate_weight));
+	  update_threadlist_entry_style(application, favorites_view, proxy);
+	}
+    }
+}
+
+
 static void
-mark_thread_cb(ThreadlistView *view, OchushaBBSThread *thread, gboolean do_mark,
-	       OchushaApplication *application)
+bookmark_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
+		   OchushaApplication *application)
+{
+  bookmark_thread(application->favorites, thread, application);
+}
+
+
+static void
+mark_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
+	       gboolean do_mark, OchushaApplication *application)
 {
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   if (do_mark)
-    info->view_flags |= BBS_THREAD_FAVORITE;
+    info->view_flags |= BBS_THREAD_MARKED;
   else
-    info->view_flags &= ~BBS_THREAD_FAVORITE;
+    info->view_flags &= ~BBS_THREAD_MARKED;
   update_threadlist_entry_style(application, view, thread);
 }
 
@@ -1835,8 +1974,8 @@ thread_title_popup_show(void)
 
   snprintf(buffer, 1024, "%s [%d/%d]",
 	   popup_thread->title,
-	   popup_thread->number_of_responses_read,
-	   popup_thread->number_of_responses_on_server);
+	   ochusha_bbs_thread_get_number_of_responses_read(popup_thread),
+	   ochusha_bbs_thread_get_number_of_responses_on_server(popup_thread));
   
   gtk_label_set_text(GTK_LABEL(title_popup_label), buffer);
 
@@ -1864,6 +2003,17 @@ item_view_required_cb(PanedNotebook *paned_notebook,
 		      OchushaApplication *application)
 {
   GtkWidget *thread_view;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
+
+  info->res_num = 0;
+  info->num_shown = 0;
+  info->last_read = NULL;
+  info->next_mark = NULL;
+  info->bookmark = NULL;
+  g_slist_free(info->backward_history);
+  info->backward_history = NULL;
+  g_slist_free(info->forward_history);
+  info->forward_history = NULL;
 
   *tab_label = get_tab_label(thread->title);
 
@@ -1878,8 +2028,10 @@ item_view_required_cb(PanedNotebook *paned_notebook,
       gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(*item_view),
 				     GTK_POLICY_AUTOMATIC,
 				     GTK_POLICY_AUTOMATIC);
+#if 0	/* themeǤˤޤ٤ */
       gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(*item_view),
 					  GTK_SHADOW_ETCHED_IN);
+#endif
       gtk_container_add(GTK_CONTAINER(*item_view), thread_view);
       gtk_widget_show(*item_view);
     }
@@ -1904,6 +2056,10 @@ free_response_source_buffer(OchushaBBSThread *thread, gpointer unused)
       info->last_read = NULL;
       info->next_mark = NULL;
       info->bookmark = NULL;
+      g_slist_free(info->backward_history);
+      info->backward_history = NULL;
+      g_slist_free(info->forward_history);
+      info->forward_history = NULL;
     }
 }
 
@@ -1959,7 +2115,7 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
       OCHU_THREADS_LEAVE();
       ochusha_async_buffer_suspend(buffer);
       ochusha_async_buffer_terminate(buffer);
-      ochusha_async_buffer_resume(buffer);
+      /* ochusha_async_buffer_resume(buffer); */
       OCHU_THREADS_ENTER();
     }
 
@@ -1984,16 +2140,32 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
   info->next_mark = NULL;
   info->bookmark = NULL;
 
-  if (!application->stick_bookmark)
-    info->bookmark_response_number = thread->number_of_responses_read;
+  g_slist_free(info->backward_history);
+  info->backward_history = NULL;
+  g_slist_free(info->forward_history);
+  info->forward_history = NULL;
 
+  if (!application->stick_bookmark)
+    info->bookmark_response_number
+      = ochusha_bbs_thread_get_number_of_responses_read(thread);
+
+#if 0
   threadlist_view_update_thread(THREADLIST_VIEW(threadlist_view),
 				thread, info->num_shown,
-				(MAX(thread->number_of_responses_on_server,
-				     thread->number_of_responses_read)
+				(MAX(ochusha_bbs_thread_get_number_of_responses_on_server(thread),
+				     ochusha_bbs_thread_get_number_of_responses_read(thread))
 				 - info->bookmark_response_number),
 				calculate_weight(thread, info,
-						 info->view_rank));
+						 info->view_rank,
+						 application->use_builtin_calculate_weight));
+#else
+  threadlist_view_update_thread(THREADLIST_VIEW(threadlist_view),
+				thread, info->num_shown,
+				(MAX(ochusha_bbs_thread_get_number_of_responses_on_server(thread),
+				     ochusha_bbs_thread_get_number_of_responses_read(thread))
+				 - info->bookmark_response_number),
+				-1);
+#endif
 
   if (info->scroller_id != 0)
     {
@@ -2058,16 +2230,11 @@ page_switched_cb(PanedNotebook *paned_notebook,
 
 #if ENABLE_MAIN_TOOLBAR
   {
-    gchar pseudo_url[PATH_MAX];
     OchushaBBSThread *thread = paned_notebook_get_item(paned_notebook, new_page);
     if (thread == NULL)
       set_current_url(application, "empty");
     else
-      {
-	snprintf(pseudo_url, PATH_MAX, "http://%s/%s",
-		 thread->board->server, ochusha_bbs_thread_get_url(thread));
-	set_current_url(application, pseudo_url);
-      }
+      set_current_url(application, ochusha_bbs_thread_get_url(thread));
   }
 #endif
 
@@ -2098,8 +2265,8 @@ page_switched_cb(PanedNotebook *paned_notebook,
 
 
 static void
-get_current_thread(OchushaApplication *application,
-		   GtkWidget **view, OchushaBBSThread **thread, IconLabel **tab_label)
+get_current_thread(OchushaApplication *application, GtkWidget **view,
+		   OchushaBBSThread **thread, IconLabel **tab_label)
 {
   PanedNotebook *paned_notebook;
 
@@ -2183,7 +2350,8 @@ background_write_response(WorkerThread *employee, gpointer job_args)
   response.content = args->message;
 
   snprintf(message, 4096, _("Posting response to %s@%s board."),
-	   args->thread->title, args->thread->board->name);
+	   args->thread->title,
+	   ochusha_bbs_thread_get_board(args->thread)->name);
   message_id = gtk_statusbar_push(args->application->statusbar,
 				  writing_netstat_id, message);
 
@@ -2195,7 +2363,7 @@ background_write_response(WorkerThread *employee, gpointer job_args)
     {
       BBSThreadGUIInfo *thread_info = ensure_bbs_thread_info(args->thread);
       BulletinBoardGUIInfo *board_info
-	= ensure_bulletin_board_info(args->thread->board, args->application);
+	= ensure_bulletin_board_info(ochusha_bbs_thread_get_board(args->thread), args->application);
       gboolean number_only
 	= (response.name[0] != '\0'
 	   && reverse_strpbrk(response.name, "0123456789") == NULL);
@@ -2280,7 +2448,8 @@ background_write_response(WorkerThread *employee, gpointer job_args)
   else
     {
       snprintf(message, 4096, _("Posting to %s@%s board failed."),
-	       args->thread->title, args->thread->board->name);
+	       args->thread->title,
+	       ochusha_bbs_thread_get_board(args->thread)->name);
 
       OCHU_THREADS_ENTER();
 
@@ -2322,6 +2491,15 @@ write_dialog_response_cb(GtkWidget *write_dialog, int response_id,
 
   MAIN_THREAD_EVENT_CALLBACK();
 
+  if (response_id == GTK_RESPONSE_REJECT)
+    {
+      editor = g_object_get_data(G_OBJECT(write_dialog), "editor");
+      response_editor_clear_response(editor);
+
+      ENTER_MAIN_EVENT_LOOP();
+      return;
+    }
+
   if (response_id != GTK_RESPONSE_OK)
     {
       guint message_id = (guint)g_object_get_data(G_OBJECT(write_dialog),
@@ -2395,9 +2573,10 @@ write_response(OchushaApplication *application)
   if (view == NULL || thread == NULL)
     return;
 
-  if (thread->flags & (OCHUSHA_BBS_THREAD_DAT_DROPPED
-		       | OCHUSHA_BBS_THREAD_STOPPED
-		       | OCHUSHA_BBS_THREAD_KAKO))
+  if (ochusha_bbs_thread_get_flags(thread)
+      & (OCHUSHA_BBS_THREAD_DAT_DROPPED
+	 | OCHUSHA_BBS_THREAD_STOPPED
+	 | OCHUSHA_BBS_THREAD_KAKO))
     return;
 
   if (!ochusha_bbs_thread_is_post_supported(thread))
@@ -2514,6 +2693,11 @@ text_search_window_query_changed_cb(TextSearchWindow *window, const gchar *key,
 
   MAIN_THREAD_EVENT_CALLBACK();
 
+  application->thread_search_direction = direction;
+  application->thread_search_enable_wrap = enable_wrap;
+  application->thread_search_match_case = match_case;
+  application->thread_search_use_regexp = use_regexp;
+
   get_current_thread(application, &widget, NULL, NULL);
 
   if (application->last_search_target_widget != widget)
@@ -2591,10 +2775,20 @@ text_search_window_find_next_cb(TextSearchWindow *window, const gchar *key,
 static void
 setup_thread_search_window(OchushaApplication *application)
 {
+  TextSearchWindow *window;
   application->thread_search_window = text_search_window_new();
-  text_search_window_set_enable_incremental_search(
-			TEXT_SEARCH_WINDOW(application->thread_search_window),
-			TRUE);
+
+  window = TEXT_SEARCH_WINDOW(application->thread_search_window);
+  text_search_window_set_enable_incremental_search(window, TRUE);
+  text_search_window_set_direction(window,
+				   application->thread_search_direction);
+  text_search_window_set_enable_wrap(window,
+				     application->thread_search_enable_wrap);
+  text_search_window_set_match_case(window,
+				    application->thread_search_match_case);
+  text_search_window_set_use_regexp(window,
+				    application->thread_search_use_regexp);
+
   g_signal_connect(application->thread_search_window, "response",
 		   G_CALLBACK(text_search_window_response_cb),
 		   application);
@@ -2605,7 +2799,7 @@ setup_thread_search_window(OchushaApplication *application)
 		   G_CALLBACK(text_search_window_find_next_cb),
 		   application);
   gtk_window_set_title(GTK_WINDOW(application->thread_search_window),
-		       _("Find in the current thread"));
+		       _("Find in the Current Thread"));
   gtk_window_set_transient_for(GTK_WINDOW(application->thread_search_window),
 			       application->top_level);
 }
@@ -2618,7 +2812,8 @@ start_thread_search(OchushaApplication *application)
     setup_thread_search_window(application);
 
   gtk_widget_show(application->thread_search_window);
-  gtk_widget_grab_focus(application->thread_search_window);
+  text_search_window_set_key_selected(
+			TEXT_SEARCH_WINDOW(application->thread_search_window));
 }
 
 
@@ -2703,6 +2898,80 @@ go_to_top_button_cb(GtkWidget *widget, OchushaApplication *application)
 }
 
 
+static void
+save_backward_thread_view_position(GtkWidget *widget, BBSThreadGUIInfo *info)
+{
+  BBSThreadView *view = BBS_THREAD_VIEW(widget);
+  info->backward_history
+    = g_slist_prepend(info->backward_history,
+		      (gpointer)bbs_thread_view_get_visible_offset(view));
+}
+
+
+static void
+go_forward_button_cb(GtkWidget *widget, OchushaApplication *application)
+{
+  GtkWidget *view;
+  OchushaBBSThread *thread;
+  BBSThreadGUIInfo *info;
+  get_current_thread(application, &view, &thread, NULL);
+
+  if (view == NULL || thread == NULL)
+    return;
+
+  MAIN_THREAD_EVENT_CALLBACK();
+  info = ensure_bbs_thread_info(thread);
+
+  if (info->forward_history != NULL)
+    {
+      int offset = (int)info->forward_history->data;
+      info->forward_history = g_slist_delete_link(info->forward_history,
+						  info->forward_history);
+      save_backward_thread_view_position(view, info);
+      bbs_thread_view_make_offset_visible(BBS_THREAD_VIEW(view), offset);
+    }
+
+  ENTER_MAIN_EVENT_LOOP();
+}
+
+
+static void
+save_forward_thread_view_position(GtkWidget *widget, BBSThreadGUIInfo *info)
+{
+  BBSThreadView *view = BBS_THREAD_VIEW(widget);
+  info->forward_history
+    = g_slist_prepend(info->forward_history,
+		      (gpointer)bbs_thread_view_get_visible_offset(view));
+}
+
+
+static void
+go_back_button_cb(GtkWidget *widget, OchushaApplication *application)
+{
+  GtkWidget *view;
+  OchushaBBSThread *thread;
+  BBSThreadGUIInfo *info;
+  get_current_thread(application, &view, &thread, NULL);
+
+  if (view == NULL || thread == NULL)
+    return;
+
+  MAIN_THREAD_EVENT_CALLBACK();
+  info = ensure_bbs_thread_info(thread);
+
+  if (info->backward_history != NULL)
+    {
+      int offset = (int)info->backward_history->data;
+      info->backward_history = g_slist_delete_link(info->backward_history,
+						   info->backward_history);
+      save_forward_thread_view_position(view, info);
+      bbs_thread_view_make_offset_visible(BBS_THREAD_VIEW(view), offset);
+    }
+
+  ENTER_MAIN_EVENT_LOOP();
+}
+
+
 static GtkWidget *
 create_write_dialog(OchushaApplication *application, OchushaBBSThread *thread)
 {
@@ -2717,15 +2986,17 @@ create_write_dialog(OchushaApplication *application, OchushaBBSThread *thread)
   GtkWidget *editor;
   ResponseEditor *response_editor;
   BBSThreadGUIInfo *thread_info = ensure_bbs_thread_info(thread);
-  BulletinBoardGUIInfo *board_info = ensure_bulletin_board_info(thread->board,
-								application);
+  BulletinBoardGUIInfo *board_info
+    = ensure_bulletin_board_info(ochusha_bbs_thread_get_board(thread),
+				 application);
   gboolean has_name = FALSE;
   gboolean has_mail = FALSE;
 
-  write_dialog = gtk_dialog_new_with_buttons(_("Write a response"),
+  write_dialog = gtk_dialog_new_with_buttons(_("Write a Response"),
 					 application->top_level,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					 GTK_STOCK_CLEAR, GTK_RESPONSE_REJECT,
 					 GTK_STOCK_OK, GTK_RESPONSE_OK,
 					 NULL);
   gtk_window_set_position(GTK_WINDOW(write_dialog), GTK_WIN_POS_CENTER);
@@ -2740,8 +3011,8 @@ create_write_dialog(OchushaApplication *application, OchushaBBSThread *thread)
   gtk_box_pack_start(GTK_BOX(title_hbox),
 		     gtk_label_new(_("@")), FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(title_hbox),
-		gtk_label_new(ochusha_bulletin_board_get_name(thread->board)),
-		FALSE, FALSE, 0);
+		     gtk_label_new(ochusha_bulletin_board_get_name(ochusha_bbs_thread_get_board(thread))),
+		     FALSE, FALSE, 0);
   board_label = gtk_label_new(_("board"));
   gtk_label_set_justify(GTK_LABEL(board_label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start(GTK_BOX(title_hbox), board_label, FALSE, FALSE, 5);
