@@ -83,7 +83,15 @@ extern void ochusha_marshal_VOID__OBJECT_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                           gpointer      invocation_hint,
                                                           gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER,POINTER,POINTER (marshal.list:15) */
+/* VOID:OBJECT,OBJECT,STRING,ENUM (marshal.list:15) */
+extern void ochusha_marshal_VOID__OBJECT_OBJECT_STRING_ENUM (GClosure     *closure,
+                                                             GValue       *return_value,
+                                                             guint         n_param_values,
+                                                             const GValue *param_values,
+                                                             gpointer      invocation_hint,
+                                                             gpointer      marshal_data);
+
+/* VOID:POINTER,POINTER,POINTER,POINTER (marshal.list:16) */
 extern void ochusha_marshal_VOID__POINTER_POINTER_POINTER_POINTER (GClosure     *closure,
                                                                    GValue       *return_value,
                                                                    guint         n_param_values,
@@ -91,7 +99,7 @@ extern void ochusha_marshal_VOID__POINTER_POINTER_POINTER_POINTER (GClosure
                                                                    gpointer      invocation_hint,
                                                                    gpointer      marshal_data);
 
-/* BOOLEAN:ENUM,INT (marshal.list:16) */
+/* BOOLEAN:ENUM,INT (marshal.list:17) */
 extern void ochusha_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -99,7 +107,7 @@ extern void ochusha_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* BOOLEAN:OBJECT (marshal.list:17) */
+/* BOOLEAN:OBJECT (marshal.list:18) */
 extern void ochusha_marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -107,7 +115,7 @@ extern void ochusha_marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* BOOLEAN:BOXED,POINTER,STRING (marshal.list:18) */
+/* BOOLEAN:BOXED,POINTER,STRING (marshal.list:19) */
 extern void ochusha_marshal_BOOLEAN__BOXED_POINTER_STRING (GClosure     *closure,
                                                            GValue       *return_value,
                                                            guint         n_param_values,
@@ -115,7 +123,7 @@ extern void ochusha_marshal_BOOLEAN__BOXED_POINTER_STRING (GClosure     *closure
                                                            gpointer      invocation_hint,
                                                            gpointer      marshal_data);
 
-/* BOOLEAN:STRING,INT,BOOLEAN,BOOLEAN,BOOLEAN (marshal.list:19) */
+/* BOOLEAN:STRING,INT,BOOLEAN,BOOLEAN,BOOLEAN (marshal.list:20) */
 extern void ochusha_marshal_BOOLEAN__STRING_INT_BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                                          GValue       *return_value,
                                                                          guint         n_param_values,
