@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.65 2004/01/05 13:54:51 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.76 2004/01/11 03:03:58 fuyu Exp $
  */
 
 #include "config.h"
@@ -201,6 +201,12 @@ terminate_on_signal(int sig)
       exit(0);
     }
 
+  if (application->sm_client_id != NULL)
+    {
+      ochusha_session_discard_states(application, application->sm_client_id);
+      application->session_subdir = NULL;
+    }
+
   if (!application->states_saved)
     ochusha_session_save_states(application);
 
@@ -255,6 +261,13 @@ ochusha_init_application(OchushaApplication *application)
       MAIN_MENU_REFRESH_BOARDLIST,
       NULL
     },
+    {
+      _("/Board/-------"),
+      NULL,
+      NULL,
+      0,
+      (char *)"<Separator>"
+    },
     {
       _("/Board/Hide Hidden Boardlist Items"),
       NULL,
@@ -304,6 +317,20 @@ ochusha_init_application(OchushaApplication *application)
       MAIN_MENU_REFRESH_THREADLIST_ALL,
       NULL
     },
+    {
+      _("/Thread/-------"),
+      NULL,
+      NULL,
+      0,
+      (char *)"<Separator>"
+    },
+    {
+      _("/Thread/Use Builtin Calculate Weight"),
+      NULL,
+      main_menu_callback,
+      MAIN_MENU_USE_BUILTIN_CALCULATE_WEIGHT,
+      (char *)"<CheckItem>",
+    },
     {
       _("/Thread/_Sort Threadlist by Weight"),
       NULL,
@@ -400,10 +427,11 @@ ochusha_init_application(OchushaApplication *application)
 		   G_CALLBACK(gtk_widget_destroyed), &application->top_level);
 #endif
 
-  gtk_window_set_title(application->top_level, _("ochusha"));
+  gtk_window_set_title(application->top_level, _("Ochusha"));
 
   gtk_window_set_icon(application->top_level,
-		      gdk_pixbuf_new_from_inline(-1, ochusha48_inline, FALSE, NULL));
+		      gdk_pixbuf_new_from_inline(-1, ochusha48_inline, FALSE,
+						 NULL));
   gtk_widget_set_size_request(GTK_WIDGET(application->top_level),
 			      application->width, application->height);
   gtk_widget_realize(GTK_WIDGET(application->top_level));
@@ -448,6 +476,11 @@ ochusha_init_application(OchushaApplication *application)
 				MAIN_MENU_HIDE_HIDDEN_BOARDLIST_ITEMS));
   gtk_check_menu_item_set_active(item,
 				 application->hide_hidden_boardlist_items);
+  item = GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget_by_action(
+				main_menu_item_factory,
+				MAIN_MENU_USE_BUILTIN_CALCULATE_WEIGHT));
+  gtk_check_menu_item_set_active(item,
+				 application->use_builtin_calculate_weight);
   item = GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget_by_action(
 				main_menu_item_factory,
 				MAIN_MENU_SORT_THREADLIST_BY_WEIGHT));
@@ -483,25 +516,25 @@ ochusha_init_application(OchushaApplication *application)
 #endif
 
   gtk_toolbar_insert_stock(application->toolbar, GTK_STOCK_GO_FORWARD,
-			   _("Go next page"),
+			   _("Go Next Page"),
 			   "go_next_page",
 			   GTK_SIGNAL_FUNC(go_forward_button_cb),
 			   application,
 			   0);
   gtk_toolbar_insert_stock(application->toolbar, GTK_STOCK_GO_BACK,
-			   _("Go next page"),
-			   "go_next_page",
+			   _("Go Previous Page"),
+			   "go_previous_page",
 			   GTK_SIGNAL_FUNC(go_back_button_cb),
 			   application,
 			   0);
   gtk_toolbar_insert_stock(application->toolbar, GTK_STOCK_REFRESH,
-			   _("Refresh current page"),
+			   _("Refresh Current Page"),
 			   "refresh_current_page",
 			   GTK_SIGNAL_FUNC(refresh_button_cb),
 			   application,
 			   -1);
   gtk_toolbar_insert_stock(application->toolbar, GTK_STOCK_CANCEL,
-			   _("Cancel current request"),
+			   _("Cancel Current Request"),
 			   "cancel_current_request",
 			   GTK_SIGNAL_FUNC(cancel_button_cb),
 			   application,
@@ -509,7 +542,7 @@ ochusha_init_application(OchushaApplication *application)
   application->url_box = GTK_COMBO(gtk_combo_new());
   gtk_toolbar_append_widget(application->toolbar,
 			    GTK_WIDGET(application->url_box),
-			    _("Input URL to show"),
+			    _("Input URL to Show"),
 			    "input_url_to_show");
 #if TRACE_MEMORY_USAGE
   g_signal_connect(G_OBJECT(application->url_box), "destroy",
@@ -611,7 +644,7 @@ ochusha_open_url(OchushaApplication *application, const gchar *url,
 {
   OchushaBulletinBoard *board;
   char *board_url = NULL;
-  OchushaBBSType bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
+  int bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
   char *thread_id = NULL;
   unsigned int from = 0;
   unsigned int to = 0;
@@ -950,9 +983,21 @@ cancel_button_cb(gpointer unused, OchushaApplication *application)
 static void
 on_quit(gpointer data, guint action, GtkWidget *widget)
 {
-  ochusha_session_save_states((OchushaApplication *)data);
+  OchushaApplication *application = (OchushaApplication *)data;
+  OCHU_THREADS_LEAVE();
+  ochusha_async_buffer_suspend_all();
+  ochusha_async_buffer_terminate_all();
+  OCHU_THREADS_ENTER();
 
-  finalize_xsmp_handlers((OchushaApplication *)data);
+  if (application->sm_client_id != NULL)
+    {
+      ochusha_session_discard_states(application, application->sm_client_id);
+      application->session_subdir = NULL;
+    }
+
+  ochusha_session_save_states(application);
+
+  finalize_xsmp_handlers(application);
 
   gtk_main_quit();
 }
@@ -995,6 +1040,7 @@ main_menu_callback(gpointer data, guint action, GtkWidget *widget)
 
     case MAIN_MENU_HIDE_HIDDEN_BOARDLIST_ITEMS:
     case MAIN_MENU_HIDE_HIDDEN_THREADS:
+    case MAIN_MENU_USE_BUILTIN_CALCULATE_WEIGHT:
     case MAIN_MENU_SORT_THREADLIST_BY_WEIGHT:
       {
 	GtkCheckMenuItem *item
@@ -1011,6 +1057,9 @@ main_menu_callback(gpointer data, guint action, GtkWidget *widget)
 	else if (action == MAIN_MENU_HIDE_HIDDEN_THREADS)
 	  application->hide_hidden_threads
 	    = gtk_check_menu_item_get_active(item);
+	else if (action == MAIN_MENU_USE_BUILTIN_CALCULATE_WEIGHT)
+	  application->use_builtin_calculate_weight
+	    = gtk_check_menu_item_get_active(item);
 	else
 	  application->sort_threadlist_by_weight
 	    = gtk_check_menu_item_get_active(item);
@@ -1120,6 +1169,8 @@ write_optional_prefs(gzFile *file, gpointer user_data)
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
 				  show_threadlist_entry_when_thread_selected);
   OUTPUT_CONFIG_ATTRIBUTE_STRING(file, application, threadlist_view_contents);
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
+				  use_builtin_calculate_weight);
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
 				  sort_threadlist_by_weight);
 
@@ -1195,6 +1246,22 @@ write_optional_prefs(gzFile *file, gpointer user_data)
   OUTPUT_CONFIG_ATTRIBUTE_STRING(file, application, last_mail);
 
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, shutdown_succeeded);
+
+  OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, threadlist_search_direction);
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
+				  threadlist_search_enable_wrap);
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
+				  threadlist_search_match_case);
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
+				  threadlist_search_use_regexp);
+
+  OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, thread_search_direction);
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
+				  thread_search_enable_wrap);
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
+				  thread_search_match_case);
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
+				  thread_search_use_regexp);
 }
 
 
@@ -1426,6 +1493,9 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
     application->threadlist_view_contents
       = G_STRDUP(OCHUSHA_DEFAULT_THREADLIST_VIEW_CONTENTS);
 
+  application->use_builtin_calculate_weight
+    = ochusha_utils_get_attribute_boolean(pref_attrs,
+					  "use_builtin_calculate_weight");
   application->sort_threadlist_by_weight
     = ochusha_utils_get_attribute_boolean(pref_attrs,
 					  "sort_threadlist_by_weight");
@@ -1579,6 +1649,32 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
 							      "last_mail");
   application->shutdown_succeeded
     = ochusha_utils_get_attribute_boolean(pref_attrs, "shutdown_succeeded");
+
+  application->threadlist_search_direction
+    = ochusha_utils_get_attribute_int(pref_attrs,
+				      "threadlist_search_direction");
+  application->threadlist_search_enable_wrap
+    = ochusha_utils_get_attribute_boolean(pref_attrs,
+					  "threadlist_search_enable_wrap");
+  application->threadlist_search_match_case
+    = ochusha_utils_get_attribute_boolean(pref_attrs,
+					  "threadlist_search_match_case");
+  application->threadlist_search_use_regexp
+    = ochusha_utils_get_attribute_boolean(pref_attrs,
+					  "threadlist_search_use_regexp");
+
+  application->thread_search_direction
+    = ochusha_utils_get_attribute_int(pref_attrs,
+				      "thread_search_direction");
+  application->thread_search_enable_wrap
+    = ochusha_utils_get_attribute_boolean(pref_attrs,
+					  "thread_search_enable_wrap");
+  application->thread_search_match_case
+    = ochusha_utils_get_attribute_boolean(pref_attrs,
+					  "thread_search_match_case");
+  application->thread_search_use_regexp
+    = ochusha_utils_get_attribute_boolean(pref_attrs,
+					  "thread_search_use_regexp");
 }
 
 
@@ -2153,7 +2249,7 @@ build_ochusha_setting_frame(OchushaApplication *application,
   gtk_widget_show(hbox);
   gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);
 
-#if 0
+#if 0	/* themeǤˤޤ٤ */
   gtk_frame_set_shadow_type(GTK_FRAME(ochusha_frame), GTK_SHADOW_ETCHED_OUT);
 #endif
   gtk_widget_show(outer_vbox);
@@ -2355,7 +2451,7 @@ build_threadlist_view_setting_frame(OchushaApplication *application,
   GtkWidget *vbox;
   GtkWidget *tmp_widget;
 
-#if 0
+#if 0	/* themeǤˤޤ٤ */
   gtk_frame_set_shadow_type(GTK_FRAME(threadlist_frame),
 			    GTK_SHADOW_ETCHED_OUT);
 #endif
@@ -2393,7 +2489,7 @@ build_threadlist_view_setting_frame(OchushaApplication *application,
   g_object_set_data(G_OBJECT(dialog), "show_threadlist_entry",
 		    tmp_widget);
 
-#if 0
+#if 0	/* themeǤˤޤ٤ */
   gtk_frame_set_shadow_type(GTK_FRAME(filter_frame), GTK_SHADOW_ETCHED_OUT);
 #endif
   vbox = gtk_vbox_new(FALSE, 0);
@@ -2427,7 +2523,7 @@ build_thread_view_setting_frame(OchushaApplication *application,
   GtkWidget *hbox;
   GtkObject *adjustment;
 
-#if 0
+#if 0	/* themeǤˤޤ٤ */
   gtk_frame_set_shadow_type(GTK_FRAME(outer_frame), GTK_SHADOW_ETCHED_OUT);
 #endif
   gtk_widget_show(outer_vbox);
@@ -2483,7 +2579,7 @@ build_thread_view_setting_frame(OchushaApplication *application,
 
 
   tmp_widget
-    = gtk_check_button_new_with_label(_("Enable Transparent a Bone."));
+    = gtk_check_button_new_with_label(_("Enable Transparent A Bone."));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
 			       application->enable_transparent_a_bone);
   gtk_widget_show(tmp_widget);
@@ -2491,7 +2587,7 @@ build_thread_view_setting_frame(OchushaApplication *application,
   g_object_set_data(G_OBJECT(dialog), "enable_transparent_a_bone",
 		    tmp_widget);
 
-  tmp_widget = gtk_check_button_new_with_label(_("Disable Popup of Response Made a Bone."));
+  tmp_widget = gtk_check_button_new_with_label(_("Disable Popup of Response Made A Bone."));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
 			       application->disable_popup_a_bone);
   gtk_widget_show(tmp_widget);
@@ -2501,7 +2597,7 @@ build_thread_view_setting_frame(OchushaApplication *application,
 
 
   thread_frame = gtk_frame_new(_("Default Setting of Thread Views"));
-#if 0
+#if 0	/* themeǤˤޤ٤ */
   gtk_frame_set_shadow_type(GTK_FRAME(thread_frame), GTK_SHADOW_ETCHED_OUT);
 #endif
 
@@ -2578,7 +2674,7 @@ build_bookmark_setting_frame(OchushaApplication *application,
   GtkTextBuffer *text_buffer;
   GtkTextIter iter;
 
-#if 0
+#if 0	/* themeǤˤޤ٤ */
   gtk_frame_set_shadow_type(GTK_FRAME(outer_frame), GTK_SHADOW_ETCHED_OUT);
 #endif
   gtk_widget_show(outer_vbox);
@@ -2618,7 +2714,7 @@ build_bookmark_setting_frame(OchushaApplication *application,
   tmp_container = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(tmp_container),
 				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-#if 0
+#if 0	/* themeǤˤޤ٤ */
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(tmp_container),
 				      GTK_SHADOW_ETCHED_IN);
 #endif
@@ -2678,7 +2774,7 @@ open_debug_console(OchushaApplication *application)
   scroll = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
 				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-#if 0
+#if 0	/* themeǤˤޤ٤ */
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
 				      GTK_SHADOW_ETCHED_IN);
 #endif
@@ -2782,7 +2878,7 @@ open_about_dialog(OchushaApplication *application)
 			 &about_dialog);
 
 	snprintf(text_buffer, 1024, "%s %s %s",
-		 _("ochusha"), _("Version"), PACKAGE_VERSION);
+		 _("Ochusha"), _("Version"), PACKAGE_VERSION);
 	label = gtk_label_new(text_buffer);
 	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_dialog)->vbox),
 			  label);
@@ -2865,7 +2961,7 @@ initialize_open_url_dialog(OchushaApplication *application)
   gtk_box_pack_start(GTK_BOX(tmp_hbox), open_url_combo, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(open_url_dialog)->vbox),
 		     tmp_hbox, FALSE, FALSE, 0);
-  open_url_in_tab_button = gtk_check_button_new_with_label(_("Open in tab"));
+  open_url_in_tab_button = gtk_check_button_new_with_label(_("Open in Tab"));
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(open_url_dialog)->vbox),
 		     open_url_in_tab_button, FALSE, FALSE, 0);
 }
