@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.52 2004/01/05 13:38:19 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.58 2004/01/11 02:13:14 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -44,10 +44,11 @@
 #include <gtk/gtk.h>
 
 
-#define ENABLE_DEBUG_CONSOLE						1
+#define ENABLE_DEBUG_CONSOLE						0
 #define ENABLE_MAIN_TOOLBAR						0
 
 #define OCHUSHA_GTKRC					"ochusha-gtkrc"
+#define OCHUSHA_BOOKMARK_XML				"bookmark.xml.gz"
 #define OCHUSHA_SESSION_XML				"session.xml.gz"
 #define OCHUSHA_INIT_SCM				"ochusha-init.scm"
 
@@ -153,14 +154,32 @@ struct _OchushaApplication
   GtkClipboard *clipboard;
   GtkClipboard *clipboard_primary;
 
+
+  /* Ϣ */
   GtkWidget *threadlist_search_window;
 
+  int threadlist_search_direction;
+  gboolean threadlist_search_enable_wrap;
+  gboolean threadlist_search_match_case;
+  gboolean threadlist_search_use_regexp;
+
+
   /* ⸡Ϣ */
   GtkWidget *thread_search_window;
   GtkWidget *last_search_target_widget;
 
+  int thread_search_direction;
+  gboolean thread_search_enable_wrap;
+  gboolean thread_search_match_case;
+  gboolean thread_search_use_regexp;
+
 
   /* İθǡ */
+  OchushaBoardCategory *my_folder;
+  OchushaBulletinBoard *favorites;
+#if 0
+  OchushaBulletinBoard *all_threads;
+#endif
   OchushaBBSTable *table;
 
 
@@ -190,6 +209,7 @@ struct _OchushaApplication
   gboolean thread_tab_always_show;
   gboolean show_threadlist_entry_when_thread_selected;
   char *threadlist_view_contents;
+  gboolean use_builtin_calculate_weight;
   gboolean sort_threadlist_by_weight;
 
   gboolean default_open_in_tab;
@@ -287,6 +307,7 @@ typedef enum
   MAIN_MENU_REFRESH_THREAD_ALL,
   MAIN_MENU_REFRESH_THREADLIST,
   MAIN_MENU_REFRESH_THREADLIST_ALL,
+  MAIN_MENU_USE_BUILTIN_CALCULATE_WEIGHT,
   MAIN_MENU_SORT_THREADLIST_BY_WEIGHT,
   MAIN_MENU_HIDE_HIDDEN_THREADS,
   MAIN_MENU_DEBUG_CONSOLE,
