@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: text_search_window.c,v 1.10 2003/12/30 21:53:28 fuyu Exp $
+ * $Id: text_search_window.c,v 1.12 2004/01/11 03:03:58 fuyu Exp $
  */
 
 #include "config.h"
@@ -305,7 +305,7 @@ text_search_window_init(TextSearchWindow *window)
 		     GTK_WIDGET(window->enable_wrap_button), FALSE, FALSE, 0);
 
   window->use_regexp_button
-    = GTK_TOGGLE_BUTTON(gtk_check_button_new_with_label(_("Regular expression")));
+    = GTK_TOGGLE_BUTTON(gtk_check_button_new_with_label(_("Regular Expression")));
   g_signal_connect_swapped(window->use_regexp_button, "toggled",
 			G_CALLBACK(text_search_window_process_query_change),
 			window);
@@ -420,7 +420,7 @@ text_search_window_show_no_match_text(TextSearchWindow *window)
 			 window->statusbar_message_id);
   window->statusbar_message_id
     = gtk_statusbar_push(window->statusbar, window->statusbar_context_id,
-			 _("No matching text"));
+			 _("No Matching Text"));
 }
 
 
@@ -594,6 +594,28 @@ text_search_window_get_enable_incremental_search(TextSearchWindow *window)
 }
 
 
+void
+text_search_window_set_direction(TextSearchWindow *window,
+				 TextSearchDirection direction)
+{
+  g_return_if_fail(IS_TEXT_SEARCH_WINDOW(window));
+  g_return_if_fail(direction == TEXT_SEARCH_DIRECTION_UP
+		   || direction == TEXT_SEARCH_DIRECTION_DOWN);
+  gtk_toggle_button_set_active(window->direction_up_button,
+			       direction == TEXT_SEARCH_DIRECTION_UP);
+}
+
+
+TextSearchDirection
+text_search_window_get_direction(TextSearchWindow *window)
+{
+  g_return_val_if_fail(IS_TEXT_SEARCH_WINDOW(window),
+		       TEXT_SEARCH_DIRECTION_DOWN);
+  return (gtk_toggle_button_get_active(window->direction_up_button)
+	  ? TEXT_SEARCH_DIRECTION_UP : TEXT_SEARCH_DIRECTION_DOWN);
+}
+
+
 void
 text_search_window_set_enable_wrap(TextSearchWindow *window, gboolean enable)
 {
@@ -673,4 +695,5 @@ text_search_window_set_key_selected(TextSearchWindow *window)
   window->statusbar_message_id = 0;
 
   gtk_editable_select_region(GTK_EDITABLE(window->entry), 0, -1);
+  gtk_widget_grab_focus(GTK_WIDGET(window->entry));
 }
