@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: text_search_window.h,v 1.4 2003/12/30 21:53:28 fuyu Exp $
+ * $Id: text_search_window.h,v 1.5 2004/01/10 19:35:26 fuyu Exp $
  */
 
 #ifndef _TEXT_SEARCH_WINDOW_H_
@@ -53,8 +53,8 @@ typedef struct _TextSearchWindowClass TextSearchWindowClass;
 
 
 typedef enum {
-  TEXT_SEARCH_DIRECTION_UP,
   TEXT_SEARCH_DIRECTION_DOWN,
+  TEXT_SEARCH_DIRECTION_UP,
 } TextSearchDirection;
 
 
@@ -116,7 +116,9 @@ void text_search_window_set_enable_incremental_search(TextSearchWindow *window,
 						      gboolean enable);
 gboolean text_search_window_get_enable_incremental_search(
 						TextSearchWindow *window);
-
+void text_search_window_set_direction(TextSearchWindow *window,
+				      TextSearchDirection direction);
+TextSearchDirection text_search_window_get_direction(TextSearchWindow *window);
 void text_search_window_set_enable_wrap(TextSearchWindow *window,
 					gboolean enable);
 gboolean text_search_window_get_enable_wrap(TextSearchWindow *window);
