@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_view.h,v 1.22 2004/01/01 23:05:58 fuyu Exp $
+ * $Id: threadlist_view.h,v 1.30 2004/01/10 19:36:38 fuyu Exp $
  */
 
 #ifndef _THREADLIST_VIEW_H_
@@ -31,6 +31,8 @@
 
 #include "ochusha.h"
 
+#include "ochusha_bbs_thread.h"
+
 #include <gtk/gtk.h>
 
 #ifdef __cplusplus
@@ -50,24 +52,24 @@ typedef struct _ThreadlistView ThreadlistView;
 typedef struct _ThreadlistViewClass ThreadlistViewClass;
 
 
+/*
+ * Υˡѥ᡼
+ */
 typedef enum
 {
-  LIST_ENTRY_FOREGROUND_NORMAL = 0,
-  LIST_ENTRY_FOREGROUND_EMPH,
-  LIST_ENTRY_FOREGROUND_STRONG,
-  LIST_ENTRY_FOREGROUND_HIDDEN,
-  NUMBER_OF_LIST_ENTRY_FOREGROUND_COLORS
-} ThreadlistEntryForeground;
-
-
-typedef enum
-{
-  LIST_ENTRY_BACKGROUND_NORMAL = 0,
-  LIST_ENTRY_BACKGROUND_EMPH,
-  LIST_ENTRY_BACKGROUND_STRONG,
-  LIST_ENTRY_BACKGROUND_HIDDEN,
-  NUMBER_OF_LIST_ENTRY_BACKGROUND_COLORS,
-} ThreadlistEntryBackground;
+  THREADLIST_SORT_BY_NONE,
+  THREADLIST_SORT_BY_BOARD,
+  THREADLIST_SORT_BY_RANK,
+  THREADLIST_SORT_BY_MARK,
+  THREADLIST_SORT_BY_TITLE,
+  THREADLIST_SORT_BY_N_RESPONSES,
+  THREADLIST_SORT_BY_N_RESPONSES_GOT,
+  THREADLIST_SORT_BY_N_RESPONSES_UNREAD,
+  THREADLIST_SORT_BY_N_RESPONSES_UNGOT,
+  THREADLIST_SORT_BY_N_RESPONSES_VERBOSE,
+  THREADLIST_SORT_BY_LAST_MODIFIED,
+  THREADLIST_SORT_BY_WEIGHT
+} ThreadlistViewSortMode;
 
 
 struct _ThreadlistView
@@ -76,6 +78,7 @@ struct _ThreadlistView
 
   GtkTreeModel *model;
 
+  GtkTreeViewColumn *board_column;
   GtkTreeViewColumn *rank_column;
   GtkTreeViewColumn *mark_column;
   GtkTreeViewColumn *title_column;
@@ -89,17 +92,21 @@ struct _ThreadlistView
   GtkTreeViewColumn *last_modified_column;
   GtkTreeViewColumn *weight_column;
 
+  GtkTreeViewColumn *current_sort_column;
+  ThreadlistViewSortMode sort_mode;
+
   GtkTreeSelection *selection;
 
   GtkTreePath *last_event_path;
 
   gboolean remove_hidden_thread;
+  gboolean editable;
   gboolean default_open_in_tab;
 
   /* Ϣ */
   gboolean (*title_match_func)(const gchar *title, gpointer search_data);
   int last_search_position;
-  GtkTreeRowReference *tail_row;
+  int number_of_rows;
 
   int count;
   int last_rank_diff;
@@ -114,16 +121,16 @@ struct _ThreadlistView
 
 
 typedef enum {
-  THREADLIST_SEARCH_DIRECTION_BACKWARD,
-  THREADLIST_SEARCH_DIRECTION_FORWARD
+  THREADLIST_SEARCH_DIRECTION_FORWARD,
+  THREADLIST_SEARCH_DIRECTION_BACKWARD
 } ThreadlistSearchDirection;
 
 
 typedef enum {
-  THREADLIST_SEARCH_ACTION_FORWARD,
   THREADLIST_SEARCH_ACTION_BACKWARD,
-  THREADLIST_SEARCH_ACTION_REGEXP_FORWARD,
+  THREADLIST_SEARCH_ACTION_FORWARD,
   THREADLIST_SEARCH_ACTION_REGEXP_BACKWARD,
+  THREADLIST_SEARCH_ACTION_REGEXP_FORWARD,
   THREADLIST_SEARCH_ACTION_TERMINATE
 } ThreadlistSearchAction;
 
@@ -145,10 +152,14 @@ struct _ThreadlistViewClass
 		      OchushaBBSThread *thread);
   void (*toggle_hide)(ThreadlistView *threadlist_view,
 		      OchushaBBSThread *thread);
+  void (*bookmark_thread)(ThreadlistView *threadlist_view,
+			  OchushaBBSThread *thread);
   void (*mark_thread)(ThreadlistView *threadlist_view,
 		      OchushaBBSThread *thread, gboolean do_mark);
   void (*reset_thread)(ThreadlistView *threadlist_view,
 		       OchushaBBSThread *thread);
+  void (*kill_thread)(ThreadlistView *threadlist_view,
+		      OchushaBBSThread *thread);
   void (*copy_thread_url)(ThreadlistView *threadlist_view,
 			  OchushaBBSThread *thread);
   gboolean (*advance_view)(ThreadlistView *threadlist_view,
@@ -168,25 +179,6 @@ struct _ThreadlistViewClass
 };
 
 
-/*
- * Υˡѥ᡼
- */
-typedef enum
-{
-  THREADLIST_SORT_BY_NONE = GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
-  THREADLIST_SORT_BY_RANK,
-  THREADLIST_SORT_BY_MARK,
-  THREADLIST_SORT_BY_TITLE,
-  THREADLIST_SORT_BY_N_RESPONSES,
-  THREADLIST_SORT_BY_N_RESPONSES_GOT,
-  THREADLIST_SORT_BY_N_RESPONSES_UNREAD,
-  THREADLIST_SORT_BY_N_RESPONSES_UNGOT,
-  THREADLIST_SORT_BY_N_RESPONSES_VERBOSE,
-  THREADLIST_SORT_BY_LAST_MODIFIED,
-  THREADLIST_SORT_BY_WEIGHT
-} ThreadlistViewSortMode;
-
-
 /*
  * ˤ륨ȥθɤѥ᡼
  */
@@ -197,6 +189,26 @@ typedef enum
 } ThreadlistEntryVisualFlags;
 
 
+typedef enum
+{
+  LIST_ENTRY_FOREGROUND_NORMAL = 0,
+  LIST_ENTRY_FOREGROUND_EMPH,
+  LIST_ENTRY_FOREGROUND_STRONG,
+  LIST_ENTRY_FOREGROUND_HIDDEN,
+  NUMBER_OF_LIST_ENTRY_FOREGROUND_COLORS
+} ThreadlistEntryForeground;
+
+
+typedef enum
+{
+  LIST_ENTRY_BACKGROUND_NORMAL = 0,
+  LIST_ENTRY_BACKGROUND_EMPH,
+  LIST_ENTRY_BACKGROUND_STRONG,
+  LIST_ENTRY_BACKGROUND_HIDDEN,
+  NUMBER_OF_LIST_ENTRY_BACKGROUND_COLORS,
+} ThreadlistEntryBackground;
+
+
 typedef struct _ThreadlistEntryVisualParameter
 {
   guint flags			: 2;	/* ThreadlistEntryVisualFlagsOR */
@@ -218,7 +230,8 @@ GtkWidget *threadlist_view_new(void);
 gboolean threadlist_view_get_remove_hidden_thread(ThreadlistView *view);
 void threadlist_view_set_remove_hidden_thread(ThreadlistView *view,
 					      gboolean remove);
-
+gboolean threadlist_view_get_editable(ThreadlistView *view);
+void threadlist_view_set_editable(ThreadlistView *view, gboolean editable);
 void threadlist_entry_set_foreground_color(ThreadlistEntryForeground color,
 					   const char *color_name);
 void threadlist_entry_set_background_color(ThreadlistEntryBackground color,
@@ -231,6 +244,7 @@ void threadlist_view_set_default_open_in_tab(ThreadlistView *view,
 void threadlist_view_set_sort_mode(ThreadlistView *view,
 				   ThreadlistViewSortMode mode);
 
+void threadlist_view_append_board(ThreadlistView *view);
 void threadlist_view_append_rank(ThreadlistView *view);
 void threadlist_view_append_mark(ThreadlistView *view);
 void threadlist_view_append_title(ThreadlistView *view);
@@ -247,8 +261,7 @@ void threadlist_view_append_weight(ThreadlistView *view);
 void threadlist_view_initialize_default_view(ThreadlistView *view);
 
 gboolean threadlist_view_open(ThreadlistView *view);
-void threadlist_view_close(ThreadlistView *view,
-			   OchushaBBSThread *last_thread);
+void threadlist_view_close(ThreadlistView *view);
 void threadlist_view_append_thread(ThreadlistView *view,
 				   OchushaBBSThread *thread,
 				   int num_shown, int num_unread,
