@@ -23,13 +23,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: xsmp.c,v 1.2 2003/12/30 17:21:46 fuyu Exp $
+ * $Id: xsmp.c,v 1.3 2004/01/06 17:20:41 fuyu Exp $
  */
 
 #include "config.h"
 
 #include "ochusha_private.h"
 #include "ochusha.h"
+#include "ochusha_async_buffer.h"
 
 #include "ochusha_ui.h"
 #include "xsmp.h"
@@ -198,8 +199,35 @@ save_yourself_cb(SmcConn smc_conn, SmPointer client_data, int save_type,
 
   g_free(prop_values);
 
+#if DEBUG_XSMP
+  fprintf(application->log_file,
+	  "save_yourself_cb: pre suspend_all()\n");
+  fflush(application->log_file);
+#endif
+
+  ochusha_async_buffer_suspend_all();
+
+#if DEBUG_XSMP
+  fprintf(application->log_file,
+	  "save_yourself_cb: post suspend_all()\n");
+  fflush(application->log_file);
+#endif
+
   ochusha_session_save_states(application);
 
+#if DEBUG_XSMP
+  fprintf(application->log_file,
+	  "save_yourself_cb: pre resume_all()\n");
+  fflush(application->log_file);
+#endif
+  ochusha_async_buffer_resume_all();
+
+#if DEBUG_XSMP
+  fprintf(application->log_file,
+	  "save_yourself_cb: post resume_all()\n");
+  fflush(application->log_file);
+#endif
+
   SmcSaveYourselfDone(smc_conn, True);
 
 #if DEBUG_XSMP
@@ -222,6 +250,9 @@ die_cb(SmcConn smc_conn, SmPointer client_data)
   SmcCloseConnection(application->smc_connection, 0, NULL);
   application->smc_connection = NULL;
 
+  ochusha_async_buffer_suspend_all();
+  ochusha_async_buffer_terminate_all();
+
   gtk_main_quit();
 }
 
