@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_table.c,v 1.20 2003/12/30 21:53:17 fuyu Exp $
+ * $Id: ochusha_bbs_table.c,v 1.23 2004/01/09 21:16:06 fuyu Exp $
  */
 
 #include "config.h"
@@ -87,6 +87,8 @@ ochusha_bbs_table_get_type(void)
 
 
 enum {
+  CATEGORY_NEW_SIGNAL,
+  BOARD_NEW_SIGNAL,
   BOARDLIST_READ_CATEGORY_ELEMENT_SIGNAL,
   BOARDLIST_WRITE_CATEGORY_ELEMENT_SIGNAL,
   BOARDLIST_READ_BOARD_ELEMENT_SIGNAL,
@@ -96,7 +98,8 @@ enum {
 
 
 static GObjectClass *parent_class = NULL;
-static int bbs_table_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
+static int bbs_table_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0 };
+
 
 static void
 ochusha_bbs_table_class_init(OchushaBBSTableClass *klass)
@@ -106,6 +109,27 @@ ochusha_bbs_table_class_init(OchushaBBSTableClass *klass)
   parent_class = g_type_class_peek_parent(klass);
   o_class->finalize = ochusha_bbs_table_finalize;
 
+  bbs_table_signals[CATEGORY_NEW_SIGNAL] =
+    g_signal_new("category_new",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(OchushaBBSTableClass, category_new),
+		 NULL, NULL,
+		 libochusha_marshal_OBJECT__STRING,
+		 OCHUSHA_TYPE_BOARD_CATEGORY, 1,
+		 G_TYPE_STRING);
+  bbs_table_signals[BOARD_NEW_SIGNAL] =
+    g_signal_new("board_new",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(OchushaBBSTableClass, board_new),
+		 NULL, NULL,
+		 libochusha_marshal_OBJECT__INT_STRING_STRING,
+		 OCHUSHA_TYPE_BULLETIN_BOARD, 3,
+		 G_TYPE_INT,
+		 G_TYPE_STRING,
+		 G_TYPE_STRING);
+
   bbs_table_signals[BOARDLIST_READ_CATEGORY_ELEMENT_SIGNAL] =
     g_signal_new("boardlist_read_category_element",
 		 G_TYPE_FROM_CLASS(klass),
@@ -151,6 +175,8 @@ ochusha_bbs_table_class_init(OchushaBBSTableClass *klass)
 		 OCHUSHA_TYPE_BULLETIN_BOARD,
 		 G_TYPE_POINTER);
 
+  klass->category_new = NULL;
+  klass->board_new = NULL;
   klass->boardlist_read_category_element = NULL;
   klass->boardlist_write_category_element = NULL;
   klass->boardlist_read_board_element = NULL;
@@ -217,6 +243,25 @@ ochusha_bbs_table_new(void)
 }
 
 
+void
+ochusha_bbs_table_prepend_category(OchushaBBSTable *table,
+				   OchushaBoardCategory *category)
+{
+  g_return_if_fail(OCHUSHA_IS_BBS_TABLE(table)
+		   && OCHUSHA_IS_BOARD_CATEGORY(category));
+
+  if (g_hash_table_lookup(table->category_table, category->name) != NULL)
+    { /* ϿѤξ */
+      table->category_list = g_slist_remove(table->category_list, category);
+    }
+
+  OCHU_OBJECT_REF(G_OBJECT(category));
+  g_hash_table_insert(table->category_table, category->name, category);
+
+  table->category_list = g_slist_prepend(table->category_list, category);
+}
+
+
 void
 ochusha_bbs_table_add_category(OchushaBBSTable *table,
 			       OchushaBoardCategory *category)
@@ -225,7 +270,9 @@ ochusha_bbs_table_add_category(OchushaBBSTable *table,
 		   && OCHUSHA_IS_BOARD_CATEGORY(category));
 
   if (g_hash_table_lookup(table->category_table, category->name) != NULL)
-    return;	/* ϿѤξ */
+    { /* ϿѤξ */
+      table->category_list = g_slist_remove(table->category_list, category);
+    }
 
   OCHU_OBJECT_REF(G_OBJECT(category));
   g_hash_table_insert(table->category_table, category->name, category);
@@ -339,7 +386,7 @@ ochusha_bbs_table_lookup_board_by_name(OchushaBBSTable *table,
 gboolean
 ochusha_bbs_table_check_url(OchushaBBSTable *table, const char *url,
 			    OchushaBulletinBoard **board_p,
-			    char **board_url_p, OchushaBBSType *bbs_type_p,
+			    char **board_url_p, int *bbs_type_p,
 			    char **thread_id_p,
 			    unsigned int *from_p, unsigned int *to_p,
 			    gboolean *is_kako_html_p)
@@ -754,7 +801,13 @@ endElementHandler(void *context, const xmlChar *name)
 		= ochusha_bbs_table_lookup_category(table, category_name);
 
 	      if (category == NULL)
-		category = ochusha_board_category_new(category_name);
+		{
+		  g_signal_emit(G_OBJECT(table),
+				bbs_table_signals[CATEGORY_NEW_SIGNAL],
+				0, category_name, &category);
+		  if (category == NULL)
+		    category = ochusha_board_category_new(category_name);
+		}
 	      else
 		OCHU_OBJECT_REF(G_OBJECT(category));
 
@@ -822,21 +875,30 @@ endElementHandler(void *context, const xmlChar *name)
 						sax_context->board_attributes,
 						"bbs_type");
 
-	      switch (bbs_type)
-		{
-		case OCHUSHA_BBS_TYPE_2CH:
-		case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-		  board = ochusha_board_2ch_new(board_name, base_url);
-		  break;
+	      g_signal_emit(G_OBJECT(table),
+			    bbs_table_signals[BOARD_NEW_SIGNAL],
+			    0, bbs_type, board_name, base_url, &board);
 
-		case OCHUSHA_BBS_TYPE_JBBS:
-		case OCHUSHA_BBS_TYPE_MACHIBBS:
-		case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-		case OCHUSHA_BBS_TYPE_MITINOKU:
-		  board = ochusha_board_jbbs_new(board_name, base_url);
-		default:
-		  /* board = NULL;  XXX ȤꤢĤ̵ */
-		  break;
+	      if (board == NULL)
+		{
+		  switch (bbs_type)
+		    {
+		    case OCHUSHA_BBS_TYPE_2CH:
+		    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
+		      board = ochusha_board_2ch_new(board_name, base_url);
+		      break;
+
+		    case OCHUSHA_BBS_TYPE_JBBS:
+		    case OCHUSHA_BBS_TYPE_MACHIBBS:
+		    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
+		    case OCHUSHA_BBS_TYPE_MITINOKU:
+		      board = ochusha_board_jbbs_new(board_name, base_url);
+		      break;
+
+		    default:
+		      /* board = NULL;  XXX ȤꤢĤ̵ */
+		      break;
+		    }
 		}
 
 	      if (board != NULL)
