@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_table.h,v 1.4 2003/12/24 03:40:50 fuyu Exp $
+ * $Id: ochusha_bbs_table.h,v 1.6 2004/01/09 21:16:06 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BBS_TABLE_H_
@@ -74,6 +74,11 @@ struct _OchushaBBSTableClass
   GObjectClass parent_class;
 
   /* Signals */
+  OchushaBoardCategory *(*category_new)(OchushaBBSTable *table,
+					const gchar *name);
+  OchushaBulletinBoard *(*board_new)(OchushaBBSTable *table,
+				     int bbs_type,
+				     const gchar *name, const char *base_url);
   void (*boardlist_read_category_element)(OchushaBBSTable *table,
 					  OchushaBoardCategory *category,
 					  GHashTable *category_attributes);
@@ -92,6 +97,8 @@ struct _OchushaBBSTableClass
 GType ochusha_bbs_table_get_type(void);
 OchushaBBSTable *ochusha_bbs_table_new(void);
 
+void ochusha_bbs_table_prepend_category(OchushaBBSTable *table,
+					OchushaBoardCategory *category);
 void ochusha_bbs_table_add_category(OchushaBBSTable *table,
 				    OchushaBoardCategory *category);
 void ochusha_bbs_table_remove_category(OchushaBBSTable *table,
@@ -113,7 +120,7 @@ OchushaBulletinBoard *ochusha_bbs_table_lookup_board_by_name(
 gboolean ochusha_bbs_table_check_url(OchushaBBSTable *table, const char *url,
 				     OchushaBulletinBoard **board_p,
 				     char **board_url_p,
-				     OchushaBBSType *bbs_type_p,
+				     int *bbs_type_p,
 				     char **thread_id_p,
 				     unsigned int *from_p, unsigned int *to_p,
 				     gboolean *is_kako_html_p);
