@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.28 2003/12/29 08:48:16 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.30 2004/01/08 19:44:59 fuyu Exp $
  */
 
 #include "config.h"
@@ -52,11 +52,6 @@
 #include <zlib.h>
 
 
-#ifndef MAP_NOCORE
-# define MAP_NOCORE	0
-#endif
-
-
 static OchushaBulletinBoardClass *parent_class = NULL;
 
 
@@ -90,10 +85,12 @@ static OchushaBBSThread *ochusha_board_2ch_lookup_thread_by_url(
 static OchushaBBSThread *ochusha_board_2ch_lookup_kako_thread_by_url(
 						OchushaBulletinBoard *board,
 						OchushaNetworkBroker *broker,
+						OchushaAsyncBuffer *buffer,
 						const char *url);
 static OchushaAsyncBuffer *ochusha_board_2ch_get_threadlist_source(
 					OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode);
 static gboolean ochusha_board_2ch_refresh_threadlist(
 						OchushaBulletinBoard *board,
@@ -500,6 +497,7 @@ ochusha_board_2ch_lookup_thread_by_url(OchushaBulletinBoard *board,
 static OchushaBBSThread *
 ochusha_board_2ch_lookup_kako_thread_by_url(OchushaBulletinBoard *board,
 					    OchushaNetworkBroker *broker,
+					    OchushaAsyncBuffer *buffer,
 					    const char *url)
 {
   OchushaBBSThread *thread = NULL;
@@ -523,6 +521,7 @@ ochusha_board_2ch_lookup_kako_thread_by_url(OchushaBulletinBoard *board,
 	  thread_2ch = OCHUSHA_THREAD_2CH(thread);
 	  ochusha_thread_2ch_set_kako_html_url(thread_2ch, url);
 	  dat_buffer = ochusha_bbs_thread_get_responses_source(thread, broker,
+					buffer,
 					OCHUSHA_NETWORK_BROKER_CACHE_AS_IS);
 	  if (dat_buffer != NULL)
 	    {
@@ -832,6 +831,7 @@ ochusha_board_2ch_set_cookie(OchushaBoard2ch *board, const char *cookie)
 static OchushaAsyncBuffer *
 ochusha_board_2ch_get_threadlist_source(OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode)
 {
   OchushaBoard2ch *board_2ch;
@@ -844,7 +844,7 @@ ochusha_board_2ch_get_threadlist_source(OchushaBulletinBoard *board,
     return NULL;
 
   board_2ch = OCHUSHA_BOARD_2CH(board);
-  return ochusha_network_broker_read_from_url(broker, url,
+  return ochusha_network_broker_read_from_url(broker, buffer, url,
 					      board_2ch->last_modified, mode);
 }
 
