@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_jbbs.c,v 1.7 2003/12/29 08:48:16 fuyu Exp $
+ * $Id: ochusha_board_jbbs.c,v 1.8 2004/01/08 19:44:59 fuyu Exp $
  */
 
 #include "config.h"
@@ -76,6 +76,7 @@ static OchushaBBSThread *ochusha_board_jbbs_thread_new(
 static OchushaAsyncBuffer *ochusha_board_jbbs_get_threadlist_source(
 					OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode);
 static gboolean ochusha_board_jbbs_refresh_threadlist(
 						OchushaBulletinBoard *board,
@@ -578,6 +579,7 @@ ochusha_board_jbbs_new(const gchar *name, const char *base_url)
 static OchushaAsyncBuffer *
 ochusha_board_jbbs_get_threadlist_source(OchushaBulletinBoard *board,
 					 OchushaNetworkBroker *broker,
+					 OchushaAsyncBuffer *buffer,
 					 OchushaNetworkBrokerCacheMode mode)
 {
   OchushaBoard2ch *board_2ch;
@@ -592,7 +594,7 @@ ochusha_board_jbbs_get_threadlist_source(OchushaBulletinBoard *board,
 	  >= PATH_MAX)
 	return NULL;
 #if 1	/* ʤǤΤasync⡼ɤǥȼԤ롣*/
-      return ochusha_network_broker_read_from_cgi(broker, url);
+      return ochusha_network_broker_read_from_cgi(broker, buffer, url);
 #else
       break;
 #endif
@@ -608,7 +610,7 @@ ochusha_board_jbbs_get_threadlist_source(OchushaBulletinBoard *board,
 
   board_2ch = OCHUSHA_BOARD_2CH(board);
 
-  return ochusha_network_broker_read_from_url(broker, url,
+  return ochusha_network_broker_read_from_url(broker, buffer, url,
 					      board_2ch->last_modified, mode);
 }
 
