@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.23 2003/12/30 21:53:17 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.27 2004/01/11 03:04:03 fuyu Exp $
  */
 
 #include "config.h"
@@ -133,14 +133,14 @@ ochusha_bulletin_board_class_init(OchushaBulletinBoardClass *klass)
 				  PROP_NAME,
 				  g_param_spec_string("name",
 						      _("Name"),
-						      _("The name of the board"),
-						      _("Unknown board"),
+						      _("The Name of the Board"),
+						      _("Unknown Board"),
 						      G_PARAM_READWRITE));
   g_object_class_install_property(o_class,
 				  PROP_BASE_URL,
 				  g_param_spec_string("base_url",
 						      _("Name"),
-						      _("The URL of the board"),
+						      _("The URL of the Board"),
 						      "http://www.2ch.net/",
 						      G_PARAM_READWRITE));
 
@@ -411,7 +411,7 @@ ochusha_bulletin_board_set_base_url(OchushaBulletinBoard *board,
 
 void
 ochusha_bulletin_board_set_bbs_type(OchushaBulletinBoard *board,
-				    OchushaBBSType bbs_type)
+				    int bbs_type)
 {
   OchushaBulletinBoardClass *klass;
   char *url = board->base_url;
@@ -549,6 +549,7 @@ ochusha_bulletin_board_lookup_bbs_thread_by_url(OchushaBulletinBoard *board,
 OchushaBBSThread *
 ochusha_bulletin_board_lookup_kako_thread_by_url(OchushaBulletinBoard *board,
 						 OchushaNetworkBroker *broker,
+						 OchushaAsyncBuffer *buffer,
 						 const char *url)
 {
   OchushaBulletinBoardClass *klass;
@@ -559,13 +560,14 @@ ochusha_bulletin_board_lookup_kako_thread_by_url(OchushaBulletinBoard *board,
   klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
   g_return_val_if_fail(klass->lookup_kako_thread_by_url != NULL, NULL);
 
-  return (*klass->lookup_kako_thread_by_url)(board, broker, url);
+  return (*klass->lookup_kako_thread_by_url)(board, broker, buffer, url);
 }
 
 
 OchushaAsyncBuffer *
 ochusha_bulletin_board_get_threadlist_source(OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode)
 {
   OchushaBulletinBoardClass *klass;
@@ -575,7 +577,7 @@ ochusha_bulletin_board_get_threadlist_source(OchushaBulletinBoard *board,
   klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
   g_return_val_if_fail(klass->get_threadlist_source != NULL, NULL);
 
-  return (*klass->get_threadlist_source)(board, broker, mode);
+  return (*klass->get_threadlist_source)(board, broker, buffer, mode);
 }
 
 
@@ -1258,6 +1260,15 @@ ochusha_bulletin_board_write_threadlist_xml(OchushaBulletinBoard *board,
 		       FALSE);
   g_return_val_if_fail(config->home != NULL, FALSE);
 
+  if (board->thread_list == NULL)
+    {
+#if 0
+      fprintf(stderr, "board(%s)'s thread_list is NULL!\n",
+	      board->base_url);
+#endif
+      return TRUE;
+    }
+
   if (subdir != NULL)
     {
       if (snprintf(pathname, PATH_MAX, "%s%s%s/%s",
