@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.h,v 1.14 2003/12/30 19:58:19 fuyu Exp $
+ * $Id: ochusha_bulletin_board.h,v 1.17 2004/01/09 17:01:50 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BULLETIN_BOARD_H_
@@ -55,7 +55,7 @@ extern "C" {
 typedef struct _OchushaBulletinBoard OchushaBulletinBoard;
 typedef struct _OchushaBulletinBoardClass OchushaBulletinBoardClass;
 
-typedef enum
+enum
 {
   OCHUSHA_BBS_TYPE_UNKNOWN = -1,
   OCHUSHA_BBS_TYPE_2CH = 0,
@@ -64,7 +64,7 @@ typedef enum
   OCHUSHA_BBS_TYPE_MACHIBBS,
   OCHUSHA_BBS_TYPE_JBBS_SHITARABA,
   OCHUSHA_BBS_TYPE_MITINOKU
-} OchushaBBSType;
+};
 
 
 struct _OchushaBulletinBoard
@@ -90,7 +90,7 @@ struct _OchushaBulletinBoard
   GHashTable *thread_table;	/* ID(DATե̾)=>(OchushaBBSThread *) */
   GSList *dropped_list;		/* DAT(OchushaBBSThread *)ꥹ */
 
-  OchushaBBSType bbs_type;	/* ǼĤμ */
+  int bbs_type;			/* ǼĤμ */
 
   gboolean hidden;		/* İ鱣 */
   gboolean killed;		/* İ¸˼ΤƤ */
@@ -124,9 +124,11 @@ struct _OchushaBulletinBoardClass
 					    const char *url);
   OchushaBBSThread *(*lookup_kako_thread_by_url)(OchushaBulletinBoard *board,
 						 OchushaNetworkBroker *broker,
+						 OchushaAsyncBuffer *buffer,
 						 const char *url);
   OchushaAsyncBuffer *(*get_threadlist_source)(OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode);
   gboolean (*refresh_threadlist)(OchushaBulletinBoard *board,
 				 OchushaAsyncBuffer *buffer,
@@ -168,7 +170,7 @@ const char *ochusha_bulletin_board_get_base_path(OchushaBulletinBoard *board);
 const char *ochusha_bulletin_board_get_id(OchushaBulletinBoard *board);
 
 void ochusha_bulletin_board_set_bbs_type(OchushaBulletinBoard *board,
-					 OchushaBBSType bbs_type);
+					 int bbs_type);
 
 OchushaBBSThread *ochusha_bulletin_board_bbs_thread_new(
 					OchushaBulletinBoard *board,
@@ -182,11 +184,13 @@ OchushaBBSThread *ochusha_bulletin_board_lookup_bbs_thread_by_url(
 OchushaBBSThread *ochusha_bulletin_board_lookup_kako_thread_by_url(
 					OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					const char *url);
 
 OchushaAsyncBuffer *ochusha_bulletin_board_get_threadlist_source(
 					OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode);
 
 gboolean ochusha_bulletin_board_refresh_threadlist(OchushaBulletinBoard *board,
