@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.50 2003/12/29 08:48:16 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.52 2004/01/08 19:44:59 fuyu Exp $
  */
 
 #include "config.h"
@@ -76,6 +76,7 @@ static void ochusha_thread_2ch_remove_cache(OchushaBBSThread *thread,
 static OchushaAsyncBuffer *ochusha_thread_2ch_get_responses_source(
 					OchushaBBSThread *thread,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode);
 static gboolean ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 						   OchushaAsyncBuffer *buffer,
@@ -152,6 +153,14 @@ ochusha_thread_2ch_class_init(OchushaThread2chClass *klass)
     = ochusha_thread_2ch_read_threadlist_element;
   b_class->write_threadlist_element
     = ochusha_thread_2ch_write_threadlist_element;
+
+  b_class->get_board = NULL;
+  b_class->get_number_of_responses_on_server = NULL;
+  b_class->get_number_of_responses_read = NULL;
+  b_class->set_number_of_responses_read = NULL;
+  b_class->get_flags = NULL;
+  b_class->set_flags = NULL;
+
   b_class->get_responses_source = ochusha_thread_2ch_get_responses_source;
   b_class->parse_responses = ochusha_thread_2ch_parse_responses;
   b_class->get_last_modified_utc = ochusha_thread_2ch_get_last_modified_utc;
@@ -525,6 +534,7 @@ ochusha_thread_2ch_get_base_path(OchushaThread2ch *thread_2ch)
 static OchushaAsyncBuffer *
 ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode)
 {
   OchushaThread2ch *thread_2ch;
@@ -540,7 +550,7 @@ ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
 	       && thread->number_of_responses_read > 0))
     mode = OCHUSHA_NETWORK_BROKER_CACHE_ONLY;
 
-  return ochusha_network_broker_read_from_url(broker,
+  return ochusha_network_broker_read_from_url(broker, buffer,
 				ochusha_thread_2ch_get_dat_url(thread_2ch),
 				thread_2ch->last_modified, mode);
 }
