@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_2ch.c,v 1.30 2004/01/05 14:09:57 fuyu Exp $
+ * $Id: ochusha_utils_2ch.c,v 1.33 2004/01/09 16:47:55 fuyu Exp $
  */
 
 #include "config.h"
@@ -287,7 +287,7 @@ parse_cgi_query(const char *query, char **board_id_p, char **thread_id_p,
  */
 gboolean
 ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
-			    OchushaBBSType *bbs_type_p,
+			    int *bbs_type_p,
 			    char **board_id_p, char **thread_id_p,
 			    unsigned int *from_p, unsigned int *to_p,
 			    gboolean *is_kako_html_p)
@@ -295,7 +295,7 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
   char *server;
   char *abs_path;
   char *path_pos;
-  OchushaBBSType bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
+  int bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
   char *thread_id = NULL;
   char *board_id = NULL;
   int from = 0;
@@ -670,7 +670,7 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 OchushaAsyncBuffer *
 ochusha_utils_2ch_get_bbsmenu(OchushaNetworkBroker *broker)
 {
-  return ochusha_network_broker_read_from_url(broker,
+  return ochusha_network_broker_read_from_url(broker, NULL,
 				broker->config->bbsmenu_url, NULL,
 				OCHUSHA_NETWORK_BROKER_CACHE_IGNORE);
 }
@@ -817,6 +817,7 @@ extract_boards(OchushaBBSTable *table, OchushaBoardCategory *category,
 	}
 
       board->killed = FALSE;
+      ochusha_bulletin_board_set_name(board, name);
       ochusha_board_category_add_board(category, board);
 
     search_next:
