@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: utils.c,v 1.17 2003/12/29 08:48:16 fuyu Exp $
+ * $Id: utils.c,v 1.18 2004/01/09 21:16:06 fuyu Exp $
  */
 
 #define OCHUSHA_UTILS_IMPLEMENTATION
@@ -308,7 +308,7 @@ ochusha_utils_url_extract_http_server(const char *url)
     }
 
   length = hostpart_tail - cur_pos;
-  if (length <= 0 || length >= MAX_HOSTNAME_LEN)
+  if (length < 0 || length >= MAX_HOSTNAME_LEN)
     {
 #if DEBUG_UTILS
       fprintf(stderr, "Couldn't parse URL: %s\n", url);
